@@ -1,3 +1,22 @@
+/*   This is part of LWIPv6
+ *   
+ *   Copyright 2004,2008,2011 Renzo Davoli University of Bologna - Italy
+ *   
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, write to the Free Software Foundation, Inc.,
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ */
+
 /*
  * Copyright (c) 2001-2003 Swedish Institute of Computer Science.
  * All rights reserved. 
@@ -35,10 +54,12 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <poll.h>
 #include <sys/types.h>
 #include <sys/uio.h>
 #include <sys/socket.h>
 
+#include "netif/etharp.h"
 
 #include "lwip/debug.h"
 
@@ -50,14 +71,10 @@
 #include "lwip/pbuf.h"
 #include "lwip/sys.h"
 
-#ifdef linux
 #include <sys/ioctl.h>
 #include <linux/if.h>
 #include <linux/if_tun.h>
 #define DEVTAP "/dev/net/tun"
-#else  /* linux */
-#define DEVTAP "/dev/tun0"
-#endif /* linux */
 
 /*-----------------------------------------------------------------------------------*/
 
@@ -73,15 +90,14 @@
 struct tunif {
   /* Add whatever per-interface state that is needed here. */
   int fd;
+	struct netif_fddata *fddata;
 };
 
 /* Forward declarations. */
-static void  tunif_input(struct netif *netif);
+static void  tunif_input(struct netif_fddata *fddata, short revents);
 static err_t tunif_output(struct netif *netif, struct pbuf *p,
 			       struct ip_addr *ipaddr);
 
-static void tunif_thread(void *data);
-
 /*-----------------------------------------------------------------------------------*/
 static int
 low_level_init(struct netif *netif, char *ifname)
@@ -100,7 +116,6 @@ low_level_init(struct netif *netif, char *ifname)
     perror("tunif_init");
    return ERR_IF;
   }
-#ifdef linux
 	{
 		struct ifreq ifr;
 		memset(&ifr, 0, sizeof(ifr));
@@ -111,11 +126,12 @@ low_level_init(struct netif *netif, char *ifname)
 			return ERR_IF;
 		}
 	}
-#endif /* Linux */
-
-  sys_thread_new(tunif_thread, netif, DEFAULT_THREAD_PRIO);
-	return ERR_OK;
 
+	if ((tunif->fddata=netif_addfd(netif,
+					tunif->fd, tunif_input, NULL, 0, POLLIN)) == NULL)
+		return ERR_IF;
+	else
+		return ERR_OK;
 }
 /*-----------------------------------------------------------------------------------*/
 /*
@@ -203,33 +219,6 @@ low_level_input(struct tunif *tunif,u16_t ifflags)
   return p;  
 }
 /*-----------------------------------------------------------------------------------*/
-static void 
-tunif_thread(void *arg)
-{
-  struct netif *netif;
-  struct tunif *tunif;
-  fd_set fdset;
-  int ret;
-  
-  netif = arg;
-  tunif = netif->state;
-  
-  while (1) {
-    FD_ZERO(&fdset);
-    FD_SET(tunif->fd, &fdset);
-
-    /* Wait for a packet to arrive. */
-    ret = select(tunif->fd + 1, &fdset, NULL, NULL, NULL);
-
-    if (ret == 1) {
-      /* Handle incoming packet. */
-      tunif_input(netif);
-    } else if (ret == -1) {
-      perror("tunif_thread: select");
-    }
-  }
-}
-/*-----------------------------------------------------------------------------------*/
 /*
  * tunif_output():
  *
@@ -264,12 +253,12 @@ tunif_output(struct netif *netif, struct pbuf *p,
  */
 /*-----------------------------------------------------------------------------------*/
 static void
-tunif_input(struct netif *netif)
+tunif_input(struct netif_fddata *fddata, short revents)
 {
+	struct netif *netif = fddata->netif;
   struct tunif *tunif;
   struct pbuf *p;
 
-
   tunif = netif->state;
   
   p = low_level_input(tunif,netif->flags);
@@ -281,6 +270,25 @@ tunif_input(struct netif *netif)
 
 	netif->input(p, netif);
 }
+
+/* cleanup: garbage collection */
+static err_t tunif_ctl(struct netif *netif, int request, void *arg)
+{
+	struct tunif *tunif = netif->state;
+
+	if (tunif) {
+
+		switch (request) {
+
+			case NETIFCTL_CLEANUP:
+				close(tunif->fd);
+
+				mem_free(tunif);
+		}
+	}
+	return ERR_OK;
+}
+
 /*-----------------------------------------------------------------------------------*/
 /*
  * tunif_init():
@@ -304,8 +312,15 @@ tunif_init(struct netif *netif)
   netif->state = tunif;
   netif->name[0] = IFNAME0;
   netif->name[1] = IFNAME1;
+	netif->link_type = NETIF_TUNIF;
 	netif->num=netif_next_num(netif,NETIF_TUNIF);
   netif->output = tunif_output;
+	netif->netifctl = tunif_ctl;
+	netif->mtu = 1500;
+	netif->hwaddr_len = 0;
+#if LWIP_NL
+	netif->type = ARPHRD_NONE;
+#endif
   
 	if (low_level_init(netif,ifname) < 0) {
 		mem_free(tunif);
