@@ -38,7 +38,7 @@
  * 
  * Author: Kieran Mansley <kjm25@cam.ac.uk>
  *
- * $Id: unixlib.c 525 2008-05-09 12:43:48Z rd235 $
+ * $Id: unixlib.c 1040 2011-12-13 08:34:27Z rd235 $
  */
 
 /*-----------------------------------------------------------------------------------*/
@@ -71,6 +71,7 @@
 #include "netif/tunif.h"
 #include "netif/tapif.h"
 #include "netif/loopif.h"
+#include "netif/slirpif.h"
 
 #include "lwip/sockets.h"
 
@@ -78,8 +79,6 @@
 #include "lwip/radvconf.h"
 #endif
 
-#define IFF_RUNNING 0x40
-
 /* #define MULTISTACKDEBUG*/
 
 static void multistack_daemon(void *argv);
@@ -90,14 +89,12 @@ extern int _nofdfake;
 
 /*static struct stack *mainstack;*/
 
-static void lwip_loopif_add(struct stack *stack);
+static void lwip_add_loopif(struct stack *stack);
 
 static void
 init_done(void *arg)
 {
 	sys_sem_t *sem = (sys_sem_t *) arg;
-	//if (!_nofdfake) /* no extra messages for umview! */
-	      //printf("unixlib: lwip init done\n");
 	sys_sem_signal(*sem);
 }
 
@@ -105,8 +102,6 @@ static void
 shutdown_done(void *arg)
 {
 	sys_sem_t *sem = (sys_sem_t *) arg;
-	//if (!_nofdfake) /* no extra messages for umview! */
-	      //printf("unixlib: lwip shutdown done\n");
 	sys_sem_signal(*sem);
 }
 
@@ -122,9 +117,7 @@ shutdown_done(void *arg)
  */
 void lwip_init(void)
 {
-	sys_sem_t sem;
-	
-	if (getenv("_INSIDE_UMVIEW_MODULE") != NULL) {
+	if (getenv("_INSIDE_VIEWOS_MODULE") != NULL) {
 		_nofdfake = 1;
 	}
 	
@@ -140,20 +133,28 @@ void lwip_init(void)
 	tcpip_init();
 }
 	
-struct stack *lwip_stack_new(void)
+#if LWIP_CAPABILITIES
+struct stack *lwip_add_stack_cap(unsigned long flags, lwip_capfun capfun)
+#else
+struct stack *lwip_add_stack(unsigned long flags)
+#endif
 {
 	sys_sem_t sem;
 	struct stack *newstack;  
 
 	/* Start the main stack */
 	sem = sys_sem_new(0);
-	newstack = tcpip_start(init_done, &sem);
+#if LWIP_CAPABILITIES
+	newstack = tcpip_start(init_done, &sem, flags, capfun);
+#else
+	newstack = tcpip_start(init_done, &sem, flags);
+#endif
 	
 	sys_sem_wait(sem);
 	sys_sem_free(sem);
 	
 	/* Add loop interface at least */       
-	lwip_loopif_add(newstack);
+	lwip_add_loopif(newstack);
 
 #ifdef MULTISTACKDEBUG
 	printf("%s: new %p\n", __func__, newstack);
@@ -162,6 +163,26 @@ struct stack *lwip_stack_new(void)
 	return newstack;
 }
 
+#if LWIP_CAPABILITIES
+struct stack *lwip_add_stack(unsigned long flags)
+{
+	return lwip_add_stack_cap(flags, NULL);
+}
+#else
+struct stack *lwip_add_stack_cap(unsigned long flags, void *must_be_null)
+{
+	if (must_be_null != NULL)
+		return NULL;
+	else
+		return lwip_add_stack_cap(flags, NULL);
+}
+#endif
+
+struct stack *lwip_stack_new(void)
+{
+	return lwip_add_stack(0);
+}
+
 struct stack *lwip_stack_get(void)
 {
     return  tcpip_stack_get();
@@ -175,7 +196,7 @@ void lwip_stack_set(struct stack *stackid)
     tcpip_stack_set(stackid);
 }
 
-void lwip_stack_free(struct stack * stackid)
+void lwip_del_stack(struct stack * stackid)
 {
 	sys_sem_t sem;
 #ifdef MULTISTACKDEBUG
@@ -190,6 +211,21 @@ void lwip_stack_free(struct stack * stackid)
 #endif
 }
 
+void lwip_stack_free(struct stack * stackid)
+{
+	lwip_del_stack(stackid);
+}
+
+unsigned long lwip_stack_flags_get(struct stack *stackid)
+{
+	return stackid->stack_flags;
+}
+
+void lwip_stack_flags_set(struct stack *stackid, unsigned long flags)
+{
+	stackid->stack_flags=flags;
+}
+
 /**
  * Shutdown the LwIPv6 Stack.
  *
@@ -219,13 +255,13 @@ static char *nullstring="";
  * @note If IPv6 Stateless Autoconfiguration Protocol is not enabled,
  *       a Link-scope IPv6 address will be assigned to the new interface.
  */
-struct netif *lwip_vdeif_add(struct stack *stack, void *arg)
+struct netif *lwip_add_vdeif(struct stack *stack, void *arg, int flags)
 {
-#if !IPv6_AUTO_CONFIGURATION
 	struct ip_addr ipaddr, netmask;
-#endif
 	struct netif *pnetif;
 	pnetif = mem_malloc(sizeof (struct netif));
+
+	pnetif->flags = flags & NETIF_ADD_FLAGS;
 	
 	if (arg == NULL) arg = nullstring;
 	if (tcpip_netif_add(stack, pnetif, arg, vdeif_init, tcpip_input, tcpip_notify) == NULL) {
@@ -233,30 +269,27 @@ struct netif *lwip_vdeif_add(struct stack *stack, void *arg)
 		return NULL;
 	}
 
-#if !IPv6_AUTO_CONFIGURATION
-	//IP6_ADDR(&ipaddr, 0xfe80,0x0,0x0,0x0,
-	//		(pnetif->hwaddr[0]<<8 |pnetif->hwaddr[1]),
-	//		(pnetif->hwaddr[2]<<8 | 0xff),
-	//		(0xfe00 | pnetif->hwaddr[3]),
-	//		(pnetif->hwaddr[4]<<8 |pnetif->hwaddr[5]));
-	//IP6_ADDR(&netmask, 0xffff,0xffff,0xffff,0xffff,0x0,0x0,0x0,0x0);
-	//netif_add_addr(pnetif,&ipaddr, &netmask);
-
-	/* Link-scope address */
-	IP6_ADDR_LINKSCOPE(&ipaddr, pnetif->hwaddr);
-	IP6_ADDR(&netmask, 0xffff,0xffff,0xffff,0xffff,0x0,0x0,0x0,0x0);
-	netif_add_addr(pnetif, &ipaddr, &netmask);
+	if
+#if IPv6_AUTO_CONFIGURATION
+		(!(flags & NETIF_FLAG_AUTOCONF))
+#else
+		(1)
 #endif
+		{
+			/* Link-scope address */
+			IP6_ADDR_LINKSCOPE(&ipaddr, pnetif->hwaddr);
+			IP6_ADDR(&netmask, 0xffff,0xffff,0xffff,0xffff,0x0,0x0,0x0,0x0);
+			netif_add_addr(pnetif, &ipaddr, &netmask);
+		}
 
 	return(pnetif);
 }
 
-#if 0
-struct netif *lwip_vdeif_add(void *arg)
+struct netif *lwip_vdeif_add(struct stack *stack, void *arg)
 {
-	return lwip_vdeif_madd(tcpip_stack_get(),arg);
+	return lwip_add_vdeif(stack, arg, NETIF_STD_FLAGS);
 }
-#endif
+
 /**
  * Creates and adds a new TAP network interface to the stack. 
  *
@@ -273,17 +306,14 @@ struct netif *lwip_vdeif_add(void *arg)
  *
  * @note You need to configure the host side of the TAP link.
  */
-struct netif *lwip_tapif_add(struct stack *stack, void *arg)
+struct netif *lwip_add_tapif(struct stack *stack, void *arg, int flags)
 {
-#if !IPv6_AUTO_CONFIGURATION
 	struct ip_addr ipaddr, netmask;
-#endif
 	struct netif *pnetif;
 	pnetif = mem_malloc(sizeof (struct netif));
-	if (pnetif == NULL) {
-		printf("VDEIF NULL!");
-		return NULL;
-	}
+
+	pnetif->flags = flags & NETIF_ADD_FLAGS;
+	pnetif->flags |= NETIF_FLAG_POINTTOPOINT;
 
 	if (arg == NULL) arg = nullstring;
 	if (tcpip_netif_add(stack, pnetif, arg, tapif_init, tcpip_input, tcpip_notify) == NULL) {
@@ -291,30 +321,26 @@ struct netif *lwip_tapif_add(struct stack *stack, void *arg)
 		return NULL;
 	}
 
-#if !IPv6_AUTO_CONFIGURATION
-	//IP6_ADDR(&ipaddr, 0xfe80,0x0,0x0,0x0,
-	//		(pnetif->hwaddr[0]<<8 |pnetif->hwaddr[1]),
-	//		(pnetif->hwaddr[2]<<8 | 0xff),
-	//		(0xfe00 | pnetif->hwaddr[3]),
-	//		(pnetif->hwaddr[4]<<8 |pnetif->hwaddr[5]));
-	//IP6_ADDR(&netmask, 0xffff,0xffff,0xffff,0xffff,0x0,0x0,0x0,0x0);
-	//netif_add_addr(pnetif,&ipaddr, &netmask);
-
-	/* Link-scope address */
-	IP6_ADDR_LINKSCOPE(&ipaddr, pnetif->hwaddr);
-	IP6_ADDR(&netmask, 0xffff,0xffff,0xffff,0xffff,0x0,0x0,0x0,0x0);
-	netif_add_addr(pnetif, &ipaddr, &netmask);
+	if
+#if IPv6_AUTO_CONFIGURATION
+		(!(flags & NETIF_FLAG_AUTOCONF))
+#else
+		(1)
 #endif
+		{
+			/* Link-scope address */
+			IP6_ADDR_LINKSCOPE(&ipaddr, pnetif->hwaddr);
+			IP6_ADDR(&netmask, 0xffff,0xffff,0xffff,0xffff,0x0,0x0,0x0,0x0);
+			netif_add_addr(pnetif, &ipaddr, &netmask);
+		}
 
 	return(pnetif);
 }
 
-#if 0
-struct netif *lwip_tapif_add(void *arg)
+struct netif *lwip_tapif_add(struct stack *stack, void *arg)
 {
-	return lwip_tapif_madd(tcpip_stack_get(),arg);
+	return lwip_add_tapif(stack, arg, NETIF_STD_FLAGS);
 }
-#endif
 
 /**
  * Creates and adds a new TUN network interface to the stack. 
@@ -330,38 +356,65 @@ struct netif *lwip_tapif_add(void *arg)
  *
  * @note You need to configure the host side of the TUN link.
  */
-struct netif *lwip_tunif_add(struct stack *stack, void *arg)
+struct netif *lwip_add_tunif(struct stack *stack, void *arg, int flags)
 {
-#if ! IPv6_AUTO_CONFIGURATION
 	struct ip_addr ipaddr, netmask;
-#endif	
 	struct netif *pnetif;
 	pnetif = mem_malloc(sizeof (struct netif));
 
+	pnetif->flags = flags & NETIF_ADD_FLAGS;
+	pnetif->flags |= NETIF_FLAG_POINTTOPOINT;
+
 	if (arg == NULL) arg = nullstring;
 	if (tcpip_netif_add(stack, pnetif, arg, tunif_init, tcpip_input, tcpip_notify) == NULL) {
 		mem_free(pnetif);
 		return NULL;
 	}
 
-	/* Link-scope address */
-#if !IPv6_AUTO_CONFIGURATION
-	IP6_ADDR_LINKSCOPE(&ipaddr, pnetif->hwaddr);
-	IP6_ADDR(&netmask, 0xffff,0xffff,0xffff,0xffff,0x0,0x0,0x0,0x0);
-	netif_add_addr(pnetif, &ipaddr, &netmask);
+	if
+#if IPv6_AUTO_CONFIGURATION
+		(!(flags & NETIF_FLAG_AUTOCONF))
+#else
+		(1)
 #endif
+		{
+			/* Link-scope address */
+			IP6_ADDR_LINKSCOPE(&ipaddr, pnetif->hwaddr);
+			IP6_ADDR(&netmask, 0xffff,0xffff,0xffff,0xffff,0x0,0x0,0x0,0x0);
+			netif_add_addr(pnetif, &ipaddr, &netmask);
+		}
 
 	return(pnetif);
 }
 
-#if 0
-struct netif *lwip_tunif_add(void *arg)
+struct netif *lwip_tunif_add(struct stack *stack, void *arg)
 {
-	return lwip_tunif_madd(tcpip_stack_get(),arg);
+	return lwip_add_tunif(stack, arg, NETIF_STD_FLAGS);
+}
+
+#ifdef LWSLIRP
+struct netif *lwip_add_slirpif(struct stack *stack, void *arg, int flags)
+{
+	struct netif *pnetif;
+	pnetif = mem_malloc(sizeof (struct netif));
+
+	pnetif->flags = flags & NETIF_ADD_FLAGS;
+	if (arg == NULL) arg = nullstring;
+	if (tcpip_netif_add(stack, pnetif, arg, slirpif_init, tcpip_input, tcpip_notify) == NULL) {
+		mem_free(pnetif);
+		return NULL;
+	}
+
+	return(pnetif);
+}
+
+struct netif *lwip_slirpif_add(struct stack *stack, void *arg)
+{
+	return lwip_add_slirpif(stack, arg, 0);
 }
 #endif
 
-static void lwip_loopif_add(struct stack *stack)
+static void lwip_add_loopif(struct stack *stack)
 {
 	struct netif *loopif;
 	struct ip_addr ipaddr, netmask;
@@ -396,10 +449,16 @@ static void lwip_loopif_add(struct stack *stack)
  *
  * @bug It doesn't check if the interface is already up.
  */
+int lwip_ifup_flags(struct netif *netif, int flags)
+{
+	netif_set_up(netif, flags);
+	return 0;
+}
+
+/* deprecated */
 int lwip_ifup(struct netif *netif)
 {
-	netif_set_up(netif);
-	return 0;
+	lwip_ifup_flags(netif, 0);
 }
 
 /**
@@ -523,10 +582,17 @@ int lwip_radv_load_configfile(struct stack *stack,void *arg)
 {
 #if IPv6_RADVCONF
 	return radv_load_configfile(stack, (char*)arg);
-#endif
+#else
 	return -1;
+#endif
 }
 
+void lwip_radv_load_config(struct stack *stack,FILE *filein)
+{
+#if IPv6_RADVCONF
+	radv_load_config(stack, filein);
+#endif
+}
 
 void lwip_thread_new(void (* thread)(void *arg), void *arg)
 {
