@@ -2,7 +2,7 @@
  *   Developed for the Ale4NET project
  *   Application Level Environment for Networking
  *
- *   Copyright 2004 Renzo Davoli University of Bologna - Italy
+ *   Copyright 2004,2011 Renzo Davoli University of Bologna - Italy
  *   
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -184,11 +184,11 @@ netbuf_copy_partial(struct netbuf *buf, void *dataptr, u16_t len, u16_t offset)
     if (offset != 0 && offset >= p->len) {
       offset -= p->len;
     } else {    
-      for(i = offset; i < p->len; ++i) {
-  ((char *)dataptr)[left] = ((char *)p->payload)[i];
-  if (++left >= len) {
-    return;
-  }
+			for(i = offset; i < p->len; ++i) {
+				((char *)dataptr)[left] = ((char *)p->payload)[i];
+				if (++left >= len) {
+					return;
+				}
       }
       offset = 0;
     }
@@ -225,7 +225,7 @@ netconn *netconn_new_with_proto_and_callback(struct stack *stack, enum netconn_t
                                    void (*callback)(struct netconn *, enum netconn_evt, u16_t len))
 {
   struct netconn *conn;
-  struct api_msg *msg;
+  struct api_msg msg;
 
   conn = memp_malloc(MEMP_NETCONN);
   if (conn == NULL) {
@@ -247,6 +247,7 @@ netconn *netconn_new_with_proto_and_callback(struct stack *stack, enum netconn_t
     memp_free(MEMP_NETCONN, conn);
     return NULL;
   }
+  conn->connected = 0;
   conn->acceptmbox = SYS_MBOX_NULL;
   conn->sem = SYS_SEM_NULL;
   conn->state = NETCONN_NONE;
@@ -254,20 +255,13 @@ netconn *netconn_new_with_proto_and_callback(struct stack *stack, enum netconn_t
   conn->callback = callback;
   conn->recv_avail = 0;
 
-  if((msg = memp_malloc(MEMP_API_MSG)) == NULL) {
-    memp_free(MEMP_NETCONN, conn);
-    return NULL;
-  }
+  msg.type = API_MSG_NEWCONN;
+  msg.msg.msg.bc.port = proto; /* misusing the port field */
+  msg.msg.conn = conn;
   
-  msg->type = API_MSG_NEWCONN;
-  msg->msg.msg.bc.port = proto; /* misusing the port field */
-  msg->msg.conn = conn;
-  
-  api_msg_post(conn->stack, msg);  
+  api_msg_post(conn->stack, &msg);  
   
   sys_mbox_fetch(conn->mbox, NULL);
-  memp_free(MEMP_API_MSG, msg);
-
   if ( conn->err != ERR_OK ) {
     memp_free(MEMP_NETCONN, conn);
     return NULL;
@@ -294,24 +288,20 @@ netconn *netconn_new_with_callback(struct stack *stack, enum netconn_type type,
 err_t
 netconn_delete(struct netconn *conn)
 {
-  struct api_msg *msg;
+  struct api_msg msg;
   void *mem;
+	//fprintf(stderr, "netconn_delete %p\n",conn);
   
   if (conn == NULL) {
     return ERR_OK;
   }
   
-  if ((msg = memp_malloc(MEMP_API_MSG)) == NULL) {
-    return ERR_MEM;
-  }
+  msg.type = API_MSG_DELCONN;
+  msg.msg.conn = conn;
   
-  msg->type = API_MSG_DELCONN;
-  msg->msg.conn = conn;
-  
-  api_msg_post(conn->stack, msg);  
+  api_msg_post(conn->stack, &msg);  
   
   sys_mbox_fetch(conn->mbox, NULL);
-  memp_free(MEMP_API_MSG, msg);
 
   /* Drain the recvmbox. */
   if (conn->recvmbox != SYS_MBOX_NULL) {
@@ -360,68 +350,76 @@ struct stack *netconn_stack(struct netconn* conn)
 }
 
 err_t
-netconn_peer(struct netconn *conn, struct ip_addr *addr,
-       u16_t *port)
+netconn_peer(struct netconn *conn, struct ip_addr *addr, u16_t *port)
 {
-  switch (conn->type) {
-  case NETCONN_RAW:
+	struct api_msg msg;
+	err_t olderr = conn->err;
+
+	if (conn == NULL) {
+		return ERR_VAL;
+	}
+	msg.type = API_MSG_PEER;
+  msg.msg.conn = conn;
+  msg.msg.err = ERR_OK;
+	msg.msg.msg.bp.ipaddr = addr;
+	msg.msg.msg.bp.port = port;
+
+	switch (conn->type) {
+		case NETCONN_RAW:
 #if LWIP_PACKET
-  case NETCONN_PACKET_RAW:
-  case NETCONN_PACKET_DGRAM:
+		case NETCONN_PACKET_RAW:
+		case NETCONN_PACKET_DGRAM:
 #endif
-    /* return an error as connecting is only a helper for upper layers */
-    return ERR_CONN;
-  case NETCONN_UDPLITE:
-  case NETCONN_UDPNOCHKSUM:
-  case NETCONN_UDP:
-    if (conn->pcb.udp == NULL ||
-  ((conn->pcb.udp->flags & UDP_FLAGS_CONNECTED) == 0))
-     return ERR_CONN;
-    *addr = (conn->pcb.udp->remote_ip);
-    *port = conn->pcb.udp->remote_port;
-    break;
-  case NETCONN_TCP:
-    if (conn->pcb.tcp == NULL)
-      return ERR_CONN;
-    *addr = (conn->pcb.tcp->remote_ip);
-    *port = conn->pcb.tcp->remote_port;
-    break;
-	default:
-		return (conn->err = ERR_ARG);
-  }
-  return (conn->err = ERR_OK);
+			/* return an error as connecting is only a helper for upper layers */
+			return ERR_CONN;
+		case NETCONN_UDPLITE:
+		case NETCONN_UDPNOCHKSUM:
+		case NETCONN_UDP:
+			api_msg_post(conn->stack, &msg);
+			sys_mbox_fetch(conn->mbox, NULL);
+			return msg.msg.err;
+		default:
+			return ERR_ARG;
+	}
 }
 
 err_t
-netconn_addr(struct netconn *conn, struct ip_addr **addr,
-       u16_t *port)
+netconn_addr(struct netconn *conn, struct ip_addr *addr, u16_t *port)
 {
-  switch (conn->type) {
-  case NETCONN_RAW:
-    *addr = &(conn->pcb.raw->local_ip);
-    *port = conn->pcb.raw->in_protocol;
-    break;
-  case NETCONN_UDPLITE:
-  case NETCONN_UDPNOCHKSUM:
-  case NETCONN_UDP:
-    *addr = &(conn->pcb.udp->local_ip);
-    *port = conn->pcb.udp->local_port;
-    break;
-  case NETCONN_TCP:
-    *addr = &(conn->pcb.tcp->local_ip);
-    *port = conn->pcb.tcp->local_port;
-    break;
-	default:
-		return (conn->err = ERR_ARG);
-  }
-  return (conn->err = ERR_OK);
+	struct api_msg msg;
+
+	if (conn == NULL) { 
+		return ERR_VAL;
+	}         
+
+	msg.type = API_MSG_PEER;
+  msg.msg.conn = conn;
+  msg.msg.err = ERR_OK;
+	msg.msg.msg.bp.ipaddr = addr;
+	msg.msg.msg.bp.port = port;
+
+	switch (conn->type) {
+		case NETCONN_PACKET_RAW:
+		case NETCONN_PACKET_DGRAM:
+			return ERR_CONN;
+		case NETCONN_RAW:
+		case NETCONN_UDPLITE:
+		case NETCONN_UDPNOCHKSUM:
+		case NETCONN_UDP:
+		case NETCONN_TCP:
+			api_msg_post(conn->stack, &msg);
+			sys_mbox_fetch(conn->mbox, NULL);
+			return msg.msg.err;
+		default:
+			return ERR_ARG;
+	}
 }
 
 err_t
 netconn_bind(struct netconn *conn, struct ip_addr *addr,
       u16_t port)
 {
-  struct api_msg *msg;
+	struct api_msg msg;
 
   if (conn == NULL) {
     return ERR_VAL;
@@ -434,18 +432,14 @@ netconn_bind(struct netconn *conn, struct ip_addr *addr,
     }
   }
   
-  if ((msg = memp_malloc(MEMP_API_MSG)) == NULL) {
-    return (conn->err = ERR_MEM);
-  }
-  msg->type = API_MSG_BIND;
-  msg->msg.conn = conn;
-  msg->msg.msg.bc.ipaddr = addr;
-  msg->msg.msg.bc.port = port;
+  msg.type = API_MSG_BIND;
+  msg.msg.conn = conn;
+  msg.msg.msg.bc.ipaddr = addr;
+  msg.msg.msg.bc.port = port;
   
-  api_msg_post(conn->stack, msg);
+  api_msg_post(conn->stack, &msg);
   
   sys_mbox_fetch(conn->mbox, NULL);
-  memp_free(MEMP_API_MSG, msg);
   return conn->err;
 }
 
@@ -454,7 +448,7 @@ err_t
 netconn_connect(struct netconn *conn, struct ip_addr *addr,
        u16_t port)
 {
-  struct api_msg *msg;
+	struct api_msg msg;
   
   if (conn == NULL) {
     return ERR_VAL;
@@ -466,40 +460,37 @@ netconn_connect(struct netconn *conn, struct ip_addr *addr,
     }
   }
   
-  if ((msg = memp_malloc(MEMP_API_MSG)) == NULL) {
-    return ERR_MEM;
-  }
-  msg->type = API_MSG_CONNECT;
-  msg->msg.conn = conn;  
-  msg->msg.msg.bc.ipaddr = addr;
-  msg->msg.msg.bc.port = port;
+  msg.type = API_MSG_CONNECT;
+  msg.msg.conn = conn;  
+  msg.msg.msg.bc.ipaddr = addr;
+  msg.msg.msg.bc.port = port;
   
-  api_msg_post(conn->stack, msg);
+  api_msg_post(conn->stack, &msg);
   
   sys_mbox_fetch(conn->mbox, NULL);
-  memp_free(MEMP_API_MSG, msg);
+	if (conn->err == ERR_OK)
+		conn->connected = 1;
+
   return conn->err;
 }
 
 err_t
 netconn_disconnect(struct netconn *conn)
 {
-  struct api_msg *msg;
+	struct api_msg msg;
   
   if (conn == NULL) {
     return ERR_VAL;
   }
 
-  if ((msg = memp_malloc(MEMP_API_MSG)) == NULL) {
-    return ERR_MEM;
-  }
-  msg->type = API_MSG_DISCONNECT;
-  msg->msg.conn = conn;  
+  msg.type = API_MSG_DISCONNECT;
+  msg.msg.conn = conn;  
   
-  api_msg_post(conn->stack, msg);
+  api_msg_post(conn->stack, &msg);
   
   sys_mbox_fetch(conn->mbox, NULL);
-  memp_free(MEMP_API_MSG, msg);
+	if (conn->err == ERR_OK)
+		conn->connected = 0;
   return conn->err;
 
 }
@@ -507,7 +498,7 @@ netconn_disconnect(struct netconn *conn)
 err_t
 netconn_listen(struct netconn *conn)
 {
-  struct api_msg *msg;
+	struct api_msg msg;
 
   if (conn == NULL) {
     return ERR_VAL;
@@ -520,16 +511,12 @@ netconn_listen(struct netconn *conn)
     }
   }
   
-  if ((msg = memp_malloc(MEMP_API_MSG)) == NULL) {
-    return (conn->err = ERR_MEM);
-  }
-  msg->type = API_MSG_LISTEN;
-  msg->msg.conn = conn;
+  msg.type = API_MSG_LISTEN;
+  msg.msg.conn = conn;
   
-  api_msg_post(conn->stack, msg);
+  api_msg_post(conn->stack, &msg);
   
   sys_mbox_fetch(conn->mbox, NULL);
-  memp_free(MEMP_API_MSG, msg);
   return conn->err;
 }
 
@@ -547,13 +534,15 @@ netconn_accept(struct netconn *conn)
   if (conn->callback)
       (*conn->callback)(conn, NETCONN_EVT_RCVMINUS, 0);
   
+	if (newconn != NULL)
+		newconn->connected = 1;
   return newconn;
 }
 
 struct netbuf *
 netconn_recv(struct netconn *conn)
 {
-  struct api_msg *msg;
+	struct api_msg msg;
   struct netbuf *buf;
   struct pbuf *p;
   u16_t len;
@@ -573,12 +562,13 @@ netconn_recv(struct netconn *conn)
   }
 
   if (conn->type == NETCONN_TCP) {
-    if (conn->pcb.tcp->state == LISTEN) {
+		/* RD1013 do not inspect pcb. use conn->connected instead */
+		if (conn->connected == 0)
+	 	{
       conn->err = ERR_CONN;
       return NULL;
     }
 
-
     buf = memp_malloc(MEMP_NETBUF);
 
     if (buf == NULL) {
@@ -615,21 +605,16 @@ netconn_recv(struct netconn *conn)
     /*buf->addr.fromaddr = NULL;*/
 
     /* Let the stack know that we have taken the data. */
-    if ((msg = memp_malloc(MEMP_API_MSG)) == NULL) {
-      conn->err = ERR_MEM;
-      return buf;
-    }
-    msg->type = API_MSG_RECV;
-    msg->msg.conn = conn;
+    msg.type = API_MSG_RECV;
+    msg.msg.conn = conn;
     if (buf != NULL) {
-      msg->msg.msg.len = buf->p->tot_len;
+      msg.msg.msg.len = buf->p->tot_len;
     } else {
-      msg->msg.msg.len = 1;
+      msg.msg.msg.len = 1;
     }
-    api_msg_post(conn->stack, msg);
+    api_msg_post(conn->stack, &msg);
 
     sys_mbox_fetch(conn->mbox, NULL);
-    memp_free(MEMP_API_MSG, msg);
   } else {
     sys_mbox_fetch(conn->recvmbox, (void **)&buf);
 		conn->recv_avail -= buf->p->tot_len;
@@ -652,28 +637,24 @@ netconn_send(struct netconn *conn, struct netbuf *buf)
 {
   struct stack *stack = conn->stack;
   
-  struct api_msg *msg;
+	struct api_msg msg;
 
   if (conn == NULL) {
     return ERR_VAL;
   }
 
+	/*
   if (conn->err != ERR_OK) {
     return conn->err;
-  }
-
-  if ((msg = memp_malloc(MEMP_API_MSG)) == NULL) {
-    return (conn->err = ERR_MEM);
-  }
+  }*/
 
   LWIP_DEBUGF(API_LIB_DEBUG, ("netconn_send: sending %d bytes\n", buf->p->tot_len));
-  msg->type = API_MSG_SEND;
-  msg->msg.conn = conn;
-  msg->msg.msg.p = buf->p;
-  api_msg_post(stack, msg);
+  msg.type = API_MSG_SEND;
+  msg.msg.conn = conn;
+  msg.msg.msg.p = buf->p;
+	api_msg_post(stack, &msg);
 
   sys_mbox_fetch(conn->mbox, NULL);
-  memp_free(MEMP_API_MSG, msg);
   return conn->err;
 }
 
@@ -682,7 +663,7 @@ netconn_write(struct netconn *conn, void *dataptr, u16_t size, u8_t copy)
 {
   struct stack *stack = conn->stack;
   
-  struct api_msg *msg;
+	struct api_msg msg;
   u16_t len;
   
   if (conn == NULL) {
@@ -700,28 +681,26 @@ netconn_write(struct netconn *conn, void *dataptr, u16_t size, u8_t copy)
     }
   }
 
-  if ((msg = memp_malloc(MEMP_API_MSG)) == NULL) {
-    return (conn->err = ERR_MEM);
-  }
-  msg->type = API_MSG_WRITE;
-  msg->msg.conn = conn;
+  msg.type = API_MSG_WRITE;
+  msg.msg.conn = conn;
 
   conn->state = NETCONN_WRITE;
   while (conn->err == ERR_OK && size > 0) {
-    msg->msg.msg.w.dataptr = dataptr;
-    msg->msg.msg.w.copy = copy;
+    msg.msg.msg.w.dataptr = dataptr;
+    msg.msg.msg.w.copy = copy;
     
     if (conn->type == NETCONN_TCP) {
-      if (tcp_sndbuf(conn->pcb.tcp) == 0) {
+			int avail;
+      while ((avail=tcp_sndbuf(conn->pcb.tcp)) == 0) {
         sys_sem_wait(conn->sem);
         if (conn->err != ERR_OK) {
           goto ret;
         }
       }
-      if (size > tcp_sndbuf(conn->pcb.tcp)) {
+      if (size > avail) {
          /* We cannot send more than one send buffer's worth of data at a
             time. */
-         len = tcp_sndbuf(conn->pcb.tcp);
+         len = avail;
       } else {
          len = size;
       }
@@ -730,8 +709,8 @@ netconn_write(struct netconn *conn, void *dataptr, u16_t size, u8_t copy)
     }
     
     LWIP_DEBUGF(API_LIB_DEBUG, ("netconn_write: writing %d bytes (%d)\n", len, copy));
-    msg->msg.msg.w.len = len;
-    api_msg_post(stack, msg);
+    msg.msg.msg.w.len = len;
+		api_msg_post(stack, &msg);
     sys_mbox_fetch(conn->mbox, NULL);    
     if (conn->err == ERR_OK) {
       dataptr = (void *)((char *)dataptr + len);
@@ -745,14 +724,13 @@ netconn_write(struct netconn *conn, void *dataptr, u16_t size, u8_t copy)
   }
   
 ret:
-  memp_free(MEMP_API_MSG, msg);
   conn->state = NETCONN_NONE;
-	/*
+	// X1004 /*
   if (conn->sem != SYS_SEM_NULL) {
     sys_sem_free(conn->sem);
     conn->sem = SYS_SEM_NULL;
   }
-	*/
+	// */
   
   return conn->err;
 }
@@ -762,20 +740,17 @@ netconn_close(struct netconn *conn)
 {
   struct stack *stack = conn->stack;
   
-  struct api_msg *msg;
+	struct api_msg msg;
 
   if (conn == NULL) {
     return ERR_VAL;
   }
-  if ((msg = memp_malloc(MEMP_API_MSG)) == NULL) {
-    return (conn->err = ERR_MEM);
-  }
 
   conn->state = NETCONN_CLOSE;
  again:
-  msg->type = API_MSG_CLOSE;
-  msg->msg.conn = conn;
-  api_msg_post(stack, msg);
+  msg.type = API_MSG_CLOSE;
+  msg.msg.conn = conn;
+	api_msg_post(stack, &msg);
   sys_mbox_fetch(conn->mbox, NULL);
   if (conn->err == ERR_MEM &&
      conn->sem != SYS_SEM_NULL) {
@@ -783,10 +758,29 @@ netconn_close(struct netconn *conn)
     goto again;
   }
   conn->state = NETCONN_NONE;
-  memp_free(MEMP_API_MSG, msg);
   return conn->err;
 }
 
+err_t
+netconn_callback(struct netconn *conn, err_t (*fun)(struct netconn *conn, void *), void *arg)
+{
+	struct api_msg msg;
+
+	if (conn == NULL) {
+		return ERR_VAL;
+	}
+
+	msg.type = API_MSG_CALLBACK;
+	msg.msg.conn = conn;
+	msg.msg.err = ERR_OK;
+	msg.msg.msg.cb.fun = fun;
+	msg.msg.msg.cb.arg = arg;
+
+	api_msg_post(conn->stack, &msg);
+	sys_mbox_fetch(conn->mbox, NULL);
+	return msg.msg.err;
+}
+
 err_t
 netconn_err(struct netconn *conn)
 {
