@@ -2,7 +2,7 @@
  *   Developed for the Ale4NET project
  *   Application Level Environment for Networking
  *   
- *   Copyright 2004,2008 Renzo Davoli University of Bologna - Italy
+ *   Copyright 2004,2008,2011 Renzo Davoli University of Bologna - Italy
  *   
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -304,7 +304,7 @@ alloc_socket(struct netconn *newconn,u16_t family)
 				int newlen=sockets_len+NUM_SOCKETS;
 				struct lwip_socket **newsockets;
 				if ((newsockets=mem_realloc(sockets,newlen*(sizeof(struct lwip_socket *)))) == NULL) {
-					free(this);
+					mem_free(this);
 					set_errno(ENOMEM);
 					sys_sem_signal(socksem);
 					return -1;
@@ -325,7 +325,7 @@ alloc_socket(struct netconn *newconn,u16_t family)
 					fd=socket(PF_INET, SOCK_DGRAM, 0);
 				if (fd < 0) {
 					sockets[i]=NULL;
-					free(this);
+					mem_free(this);
 					set_errno(EIO);
 					return -1;
 				} 
@@ -339,6 +339,79 @@ alloc_socket(struct netconn *newconn,u16_t family)
 }
 
 
+/* syncronous access to pcb data */
+
+#define OPT_SETVALUE   1
+#define OPT_GETVALUE   2
+#define OPT_SETBITS    3
+#define OPT_CLRBITS    4
+#define OPT_GETMASKED  5
+
+#define OPT_SO_OPTIONS 1
+#define OPT_FLAGS      2
+#define OPT_TOS        3
+#define OPT_TTL        4
+#define OPT_KEEPALIVE  5
+#define OPT_CHECKSUMOFFSET  6
+
+#define opfield(op,field) (((op) << 5) | (field))
+struct opt_data {
+	u16_t opfieldtag;
+	u32_t *value;
+};
+
+static err_t sync_pcb_access(struct netconn *conn, void *arg)
+{
+	struct opt_data *data = arg;
+	switch (data->opfieldtag) {
+		case opfield(OPT_SETBITS, OPT_SO_OPTIONS): 
+			conn->pcb.common->so_options |= *data->value; break;
+		case opfield(OPT_CLRBITS, OPT_SO_OPTIONS): 
+			conn->pcb.common->so_options &= ~(*data->value); break;
+		case opfield(OPT_GETMASKED, OPT_SO_OPTIONS): 
+			*data->value = conn->pcb.common->so_options & (*data->value); break;
+
+		case opfield(OPT_SETBITS, OPT_FLAGS): 
+			conn->pcb.common->so_options |= *data->value; break;
+		case opfield(OPT_CLRBITS, OPT_FLAGS): 
+			conn->pcb.common->so_options &= ~(*data->value); break;
+		case opfield(OPT_GETMASKED, OPT_FLAGS): 
+			*data->value = conn->pcb.common->so_options & (*data->value); break;
+
+		case opfield(OPT_SETVALUE, OPT_TOS):
+			conn->pcb.tcp->tos = (*data->value); break;
+		case opfield(OPT_GETVALUE, OPT_TOS):
+			(*data->value) = conn->pcb.tcp->tos; break;
+
+		case opfield(OPT_SETVALUE, OPT_TTL):
+			conn->pcb.tcp->ttl = (*data->value); break;
+		case opfield(OPT_GETVALUE, OPT_TTL):
+			(*data->value) = conn->pcb.tcp->ttl; break;
+
+		case opfield(OPT_SETVALUE, OPT_KEEPALIVE):
+			conn->pcb.tcp->keepalive = (*data->value); break;
+		case opfield(OPT_GETVALUE, OPT_KEEPALIVE):
+			(*data->value) = conn->pcb.tcp->keepalive; break;
+
+		case opfield(OPT_SETVALUE, OPT_CHECKSUMOFFSET):
+			conn->pcb.raw->so_options |= SOF_IPV6_CHECKSUM;
+			conn->pcb.raw->checksumoffset = (*data->value); break;
+	}
+	return ERR_OK;
+}
+
+static u32_t pcb_access(struct netconn *conn, u8_t op, u8_t field, u32_t value)
+{
+	struct opt_data data = {
+		.opfieldtag = opfield(op,field),
+		.value = &value
+	};
+	netconn_callback(conn, sync_pcb_access, &data);
+	return value;
+}
+
+/* ----- end of syncronous access to pcb data functions ---- */
+
 	int
 lwip_accept(int s, struct sockaddr *addr, socklen_t *addrlen)
 {
@@ -367,30 +440,32 @@ lwip_accept(int s, struct sockaddr *addr, socklen_t *addrlen)
 	/* get the IP address and port of the remote host */
 	netconn_peer(newconn, &naddr, &port);
 
-	if (sock->family == PF_INET) {
-		struct sockaddr_in sin;
-		memset(&sin, 0, sizeof(sin));
-		sin.sin_family = sock->family;
-		sin.sin_port = htons(port);
-		/*memcpy(&(sin.sin_addr.s_addr),&(naddr.addr[3]),sizeof(sin.sin_addr.s_addr));*/
-		SOCK_IP46_CONV(&(sin.sin_addr.s_addr),&(naddr));
+	if (addr != NULL) {
+		if (sock->family == PF_INET) {
+			struct sockaddr_in sin;
+			memset(&sin, 0, sizeof(sin));
+			sin.sin_family = sock->family;
+			sin.sin_port = htons(port);
+			/*memcpy(&(sin.sin_addr.s_addr),&(naddr.addr[3]),sizeof(sin.sin_addr.s_addr));*/
+			SOCK_IP46_CONV(&(sin.sin_addr.s_addr),&(naddr));
 
-		if (*addrlen > sizeof(sin))
-			*addrlen = sizeof(sin);
+			if (*addrlen > sizeof(sin))
+				*addrlen = sizeof(sin);
 
-		memcpy(addr, &sin, *addrlen);
-	}           
-	else {      
-		struct sockaddr_in6 sin;
-		memset(&sin, 0, sizeof(sin));
-		sin.sin6_family = sock->family;
-		sin.sin6_port = htons(port);
-		memcpy(&(sin.sin6_addr),&(naddr.addr),sizeof(sin.sin6_addr));
+			memcpy(addr, &sin, *addrlen);
+		}           
+		else {      
+			struct sockaddr_in6 sin;
+			memset(&sin, 0, sizeof(sin));
+			sin.sin6_family = sock->family;
+			sin.sin6_port = htons(port);
+			memcpy(&(sin.sin6_addr),&(naddr.addr),sizeof(sin.sin6_addr));
 
-		if (*addrlen > sizeof(sin))
-			*addrlen = sizeof(sin);
+			if (*addrlen > sizeof(sin))
+				*addrlen = sizeof(sin);
 
-		memcpy(addr, &sin, *addrlen);
+			memcpy(addr, &sin, *addrlen);
+		}
 	}
 
 	/* set by the EVT_ACCEPTPLUS event */
@@ -446,6 +521,14 @@ lwip_bind(int s, struct sockaddr *name, socklen_t namelen)
 				local_port = ((struct sockaddr_in6 *)name)->sin6_port;
 			}
 
+#if LWIP_CAPABILITIES
+		struct stack *stack=netconn_stack(sock->conn);
+		if (stack->stack_capfun && (stack->stack_capfun()&LWIP_CAP_NET_BIND_SERVICE) == 0) {
+			set_errno(EPERM);
+			return -1;
+		}
+#endif
+
 		LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_bind(%d, addr=", s));
 		ip_addr_debug_print(SOCKETS_DEBUG, &local_addr);
 		LWIP_DEBUGF(SOCKETS_DEBUG, (" port=%u)\n", ntohs(local_port)));
@@ -505,7 +588,7 @@ lwip_close(int s)
 		close(sock->fdfake);
 	set_lwip_sockmap(sock->fdfake, -1);
 	sock_set_errno(sock, err);
-	free(sock);
+	mem_free(sock);
 	sys_sem_signal(socksem);
 	return err;
 }
@@ -730,10 +813,14 @@ lwip_recvfrom(int s, void *mem, int len, unsigned int flags,
 #endif
 		}
 
+		if (flags & MSG_PEEK) {
+			sock->lastdata = buf;
+			/* lastoffset does not change, usually it is 0 and it keeps its value */
+		}
 		/* If this is a TCP socket, check if there is data left in the
 		   buffer. If so, it should be saved in the sock structure for next
 		   time around. */
-		if (netconn_type(sock->conn) == NETCONN_TCP && buflen - copylen > 0) {
+		else if (netconn_type(sock->conn) == NETCONN_TCP && buflen - copylen > 0) {
 			sock->lastdata = buf;
 			sock->lastoffset += copylen;
 		} else {
@@ -743,7 +830,10 @@ lwip_recvfrom(int s, void *mem, int len, unsigned int flags,
 		}
 
 		sock_set_errno(sock, 0);
-		return copylen;
+		if (flags & MSG_TRUNC) 
+			return buflen;
+		else
+			return copylen;
 	}
 }
 
@@ -763,10 +853,14 @@ ssize_t lwip_recvmsg(int fd, struct msghdr *msg, int flags)
 {
 	msg->msg_controllen=0;
 	if (msg->msg_iovlen == 1) {
-		return lwip_recvfrom(fd, msg->msg_iov->iov_base,msg->msg_iov->iov_len,flags,
+		ssize_t ret=lwip_recvfrom(fd, msg->msg_iov->iov_base,msg->msg_iov->iov_len,flags,
 				msg->msg_name,&(msg->msg_namelen));
+		if (ret > msg->msg_iov->iov_len) 
+			msg->msg_flags |= MSG_TRUNC;
+		return ret;
+
 	} else {
-		struct iovec liovec,*msg_iov;
+		struct iovec *msg_iov;
 		size_t msg_iovlen;
 		unsigned int i,totalsize;
 		size_t size;
@@ -775,9 +869,10 @@ ssize_t lwip_recvmsg(int fd, struct msghdr *msg, int flags)
 		msg_iovlen=msg->msg_iovlen;
 		for (i=0,totalsize=0;i<msg_iovlen;i++)
 			totalsize += msg_iov[i].iov_len;
-		liovec.iov_base=lbuf=alloca(totalsize);
-		liovec.iov_len=totalsize;
-		size=lwip_recvfrom(fd, liovec.iov_base, liovec.iov_len, flags, msg->msg_name,&(msg->msg_namelen));
+		lbuf=alloca(totalsize);
+		size=lwip_recvfrom(fd, lbuf, totalsize, flags, msg->msg_name,&(msg->msg_namelen));
+		if (size > totalsize)
+			msg->msg_flags |= MSG_TRUNC;
 		for (i=0;size > 0 && i<msg_iovlen;i++) {
 			int qty=(size > msg_iov[i].iov_len)?msg_iov[i].iov_len:size;
 			memcpy(msg_iov[i].iov_base,lbuf,qty);
@@ -929,7 +1024,7 @@ ssize_t lwip_sendmsg(int fd, struct msghdr *msg, int flags)
 		return lwip_sendto(fd, msg->msg_iov->iov_base,msg->msg_iov->iov_len,flags,
 				msg->msg_name,msg->msg_namelen);
 	} else {
-		struct iovec liovec,*msg_iov;
+		struct iovec *msg_iov;
 		size_t msg_iovlen;
 		unsigned int i,totalsize;
 		size_t size;
@@ -938,15 +1033,13 @@ ssize_t lwip_sendmsg(int fd, struct msghdr *msg, int flags)
 		msg_iovlen=msg->msg_iovlen;
 		for (i=0,totalsize=0;i<msg_iovlen;i++)
 			totalsize += msg_iov[i].iov_len;
-		liovec.iov_base=lbuf=alloca(totalsize);
-		liovec.iov_len=size=totalsize;
 		for (i=0;size > 0 && i<msg_iovlen;i++) {
 			int qty=msg_iov[i].iov_len;
 			memcpy(lbuf,msg_iov[i].iov_base,qty);
 			lbuf+=qty;
 			size-=qty;
 		}
-		size=lwip_sendto(fd, liovec.iov_base, liovec.iov_len, flags, msg->msg_name,msg->msg_namelen);
+		size=lwip_sendto(fd, lbuf, totalsize, flags, msg->msg_name,msg->msg_namelen);
 		return size;
 	}
 }
@@ -957,6 +1050,11 @@ lwip_msocket(struct stack *stack, int domain, int type, int protocol)
 	struct netconn *conn;
 	int i;
 
+	if (stack==NULL) {
+		set_errno(ENONET);
+		return -1;
+	}
+
 	if (domain != PF_INET && domain != PF_INET6
 #if LWIP_NL
 			&& domain != PF_NETLINK
@@ -968,8 +1066,17 @@ lwip_msocket(struct stack *stack, int domain, int type, int protocol)
 		set_errno(EAFNOSUPPORT);
 		return -1;
 	}
+	
+#if LWIP_CAPABILITIES
+	if (domain == PF_PACKET ||
+			(domain == PF_INET || domain == PF_INET6) && type == SOCK_RAW) {
+		if (stack->stack_capfun && (stack->stack_capfun()&LWIP_CAP_NET_RAW)==0) {
+			set_errno(EPERM);
+			return -1;
+		}
+	}
+#endif
 
-	//#if LWIP_NL || LWIP_PACKET
 	switch(domain) {
 #if LWIP_NL
 		case PF_NETLINK:
@@ -981,7 +1088,7 @@ lwip_msocket(struct stack *stack, int domain, int type, int protocol)
 						return -1;
 					} else {
 						LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_socket(%s,XXX, %d) = ", domain_name(domain), protocol));
-						conn = netlink_open(stack, type,protocol);
+						conn = netlink_open(stack, type, protocol);
 					}
 					break;
 				default:
@@ -1012,7 +1119,6 @@ lwip_msocket(struct stack *stack, int domain, int type, int protocol)
 
 		case PF_INET:
 		case PF_INET6:
-			//#endif 
 
 			/* create a netconn */
 			switch (type) {
@@ -1036,10 +1142,7 @@ lwip_msocket(struct stack *stack, int domain, int type, int protocol)
 			break;
 		default:
 			LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_socket(%d/UNKNOWN, %d, %d) = -1\n", domain, type, protocol));
-
-			//#if LWIP_NL
 	}
-	//#endif
 
 	if (!conn) {
 		LWIP_DEBUGF(SOCKETS_DEBUG, ("-1 / ENOBUFS (could not create netconn)\n"));
@@ -1122,7 +1225,7 @@ static struct um_sel_wait *um_sel_rec_del(struct um_sel_wait *p,void *arg,int fd
 	else {
 		struct um_sel_wait *next=um_sel_rec_del(p->next,arg,fd);
 		if (p->arg == arg && p->fd == fd) {
-			free(p);
+			mem_free(p);
 			return(next);
 		} else {
 			p->next=next;
@@ -1147,7 +1250,7 @@ static struct um_sel_wait *um_sel_rec_signal(struct um_sel_wait *p,int fd, int e
 			struct um_sel_wait *next=um_sel_rec_del(p->next,p->arg,fd);
 			//printf("UMSELECT SIGNALED %d\n",fd);
 			p->cb(p->arg);
-			free(p);
+			mem_free(p);
 			return um_sel_rec_signal(next,fd,events);
 		} else {
 			p->next=um_sel_rec_signal(p->next,fd,events);
@@ -1342,7 +1445,7 @@ lwip_getpeername (int s, struct sockaddr *name, socklen_t *namelen)
 lwip_getsockname (int s, struct sockaddr *name, socklen_t *namelen)
 {
 	struct lwip_socket *sock;
-	struct ip_addr *naddr;
+	struct ip_addr naddr;
 	u16_t port;
 
 	sock = get_socket(s);
@@ -1358,11 +1461,16 @@ lwip_getsockname (int s, struct sockaddr *name, socklen_t *namelen)
 	else 
 #endif
 	{
+		int err;
 		/* get the IP address and port of the remote host */
-		netconn_addr(sock->conn, &naddr, &port);
+		err = netconn_addr(sock->conn, &naddr, &port);
+		if (err != ERR_OK) {
+			set_errno(EINVAL);
+			return -1;
+		}
 
 		LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_getsockname(%d, addr=", s));
-		ip_addr_debug_print(SOCKETS_DEBUG, naddr);
+		ip_addr_debug_print(SOCKETS_DEBUG, &naddr);
 		LWIP_DEBUGF(SOCKETS_DEBUG, (" port=%d)\n", port));
 
 		if (sock->family == PF_INET) {
@@ -1371,7 +1479,7 @@ lwip_getsockname (int s, struct sockaddr *name, socklen_t *namelen)
 			sin.sin_family = PF_INET;
 			sin.sin_port = htons(port);
 			/*memcpy(&(sin.sin_addr.s_addr),&(naddr.addr),sizeof(sin.sin_addr.s_addr));*/
-			SOCK_IP46_CONV(&(sin.sin_addr.s_addr),naddr);
+			SOCK_IP46_CONV(&(sin.sin_addr.s_addr),&naddr);
 
 			if (*namelen > sizeof(sin))
 				*namelen = sizeof(sin);
@@ -1384,7 +1492,7 @@ lwip_getsockname (int s, struct sockaddr *name, socklen_t *namelen)
 			memset(&sin, 0, sizeof(sin));
 			sin.sin6_family = PF_INET6;
 			sin.sin6_port = htons(port);
-			memcpy(&(sin.sin6_addr),naddr,sizeof(sin.sin6_addr));
+			memcpy(&(sin.sin6_addr),&naddr,sizeof(sin.sin6_addr));
 
 			if (*namelen > sizeof(sin))
 				*namelen = sizeof(sin);
@@ -1407,9 +1515,15 @@ lwip_getsockopt (int s, int level, int optname, void *optval, socklen_t *optlen)
 		set_errno(EBADF);
 		return -1;
 	}
+
+	if( NULL == optval || NULL == optlen ) {
+		sock_set_errno( sock, EFAULT );
+		return -1;
+	}
+
 #if LWIP_NL
 	if(sock->family == PF_NETLINK) {
-		int err=netlink_getsockopt(sock, level, optname, optval, optlen); 
+		int err=netlink_getsockopt(sock->conn, level, optname, optval, optlen); 
 		if (err != 0) {
 			sock_set_errno(sock, err);
 			return -1;
@@ -1418,11 +1532,6 @@ lwip_getsockopt (int s, int level, int optname, void *optval, socklen_t *optlen)
 	}
 #endif
 
-	if( NULL == optval || NULL == optlen ) {
-		sock_set_errno( sock, EFAULT );
-		return -1;
-	}
-
 	/* Do length and type checks for the various options first, to keep it readable. */
 	switch( level ) {
 
@@ -1439,6 +1548,9 @@ lwip_getsockopt (int s, int level, int optname, void *optval, socklen_t *optlen)
 					/* UNIMPL case SO_OOBINLINE: */
 					/* UNINPL case SO_RCVBUF: */
 					/* UNINPL case SO_SNDBUF: */
+					/* FAKE SO_RCVBUF, SO_SNDBUF */
+				case SO_RCVBUF:
+				case SO_SNDBUF:
 					/* UNIMPL case SO_RCVLOWAT: */
 					/* UNIMPL case SO_SNDLOWAT: */
 #if SO_REUSE
@@ -1532,7 +1644,8 @@ lwip_getsockopt (int s, int level, int optname, void *optval, socklen_t *optlen)
 				case SO_REUSEPORT:
 #endif /* SO_REUSE */
 					/*case SO_USELOOPBACK: UNIMPL */
-					*(int*)optval = sock->conn->pcb.tcp->so_options & so_map[optname];
+					//*(int*)optval = sock->conn->pcb.tcp->so_options & so_map[optname];
+					*(int*)optval = pcb_access(sock->conn,OPT_GETMASKED,OPT_SO_OPTIONS,so_map[optname]);
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_getsockopt(%d, SOL_SOCKET, optname=0x%x, ..) = %s\n", s, optname, (*(int*)optval?"on":"off")));
 					break;
 
@@ -1561,6 +1674,12 @@ lwip_getsockopt (int s, int level, int optname, void *optval, socklen_t *optlen)
 					sock->err = 0;
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_getsockopt(%d, SOL_SOCKET, SO_ERROR) = %d\n", s, *(int *)optval));
 					break;
+
+					/*fake SO_RCVBUF, SO_SNDBUF, return the max value */
+				case SO_RCVBUF:
+				case SO_SNDBUF:
+					*(int *)optval = 262144;
+					break;
 			}  /* switch */
 			break;
 
@@ -1568,15 +1687,19 @@ lwip_getsockopt (int s, int level, int optname, void *optval, socklen_t *optlen)
 		case IPPROTO_IP:
 			switch( optname ) {
 				case IP_TTL:
-					*(int*)optval = sock->conn->pcb.tcp->ttl;
+					//*(int*)optval = sock->conn->pcb.tcp->ttl;
+					*(int*)optval = pcb_access(sock->conn,OPT_GETVALUE,OPT_TTL,0);
+
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_getsockopt(%d, IPPROTO_IP, IP_TTL) = %d\n", s, *(int *)optval));
 					break;
 				case IP_TOS:
-					*(int*)optval = sock->conn->pcb.tcp->tos;
+					//*(int*)optval = sock->conn->pcb.tcp->tos;
+					*(int*)optval = pcb_access(sock->conn,OPT_GETVALUE,OPT_TOS,0);
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_getsockopt(%d, IPPROTO_IP, IP_TOS) = %d\n", s, *(int *)optval));
 					break;
 				case IP_HDRINCL:
-					*(int*)optval = (sock->conn->pcb.tcp->so_options & SOF_HDRINCL);
+					//*(int*)optval = (sock->conn->pcb.tcp->so_options & SOF_HDRINCL);
+					*(int*)optval = pcb_access(sock->conn,OPT_GETMASKED,OPT_SO_OPTIONS,SOF_HDRINCL);
 					break;
 			}  /* switch */
 			break;
@@ -1585,18 +1708,19 @@ lwip_getsockopt (int s, int level, int optname, void *optval, socklen_t *optlen)
 		case IPPROTO_TCP:
 			switch( optname ) {
 				case TCP_NODELAY:
-					*(int*)optval = (sock->conn->pcb.tcp->flags & TF_NODELAY);
+					//*(int*)optval = (sock->conn->pcb.tcp->flags & TF_NODELAY);
+					*(int*)optval = pcb_access(sock->conn,OPT_GETMASKED,OPT_FLAGS,TF_NODELAY);
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_getsockopt(%d, IPPROTO_TCP, TCP_NODELAY) = %s\n", s, (*(int*)optval)?"on":"off") );
 					break;
 				case TCP_KEEPALIVE:
-					*(int*)optval = sock->conn->pcb.tcp->keepalive;
+					//*(int*)optval = sock->conn->pcb.tcp->keepalive;
+					*(int*)optval = pcb_access(sock->conn,OPT_GETVALUE,OPT_KEEPALIVE,0);
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_getsockopt(%d, IPPROTO_IP, TCP_KEEPALIVE) = %d\n", s, *(int *)optval));
 					break;
 			}  /* switch */
 			break;
 	}
 
-
 	sock_set_errno(sock, err);
 	return err ? -1 : 0;
 }
@@ -1607,7 +1731,6 @@ lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_t op
 	struct lwip_socket *sock;
 	int err = 0;
 
-	//printf("lwip_setsockopt %d %d\n",level,optname);
 	sock = get_socket(s);
 	if (!sock) {
 		set_errno(EBADF);
@@ -1621,7 +1744,7 @@ lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_t op
 
 #if LWIP_NL
 	if(sock->family == PF_NETLINK) {
-		int err=netlink_setsockopt(sock, level, optname, optval, optlen);
+		int err=netlink_setsockopt(sock->conn, level, optname, optval, optlen);
 		if (err != 0) {
 			sock_set_errno(sock, err);
 			return -1;
@@ -1642,10 +1765,13 @@ lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_t op
 					/* UNIMPL case SO_DEBUG: */
 					/* UNIMPL case SO_DONTROUTE: */
 				case SO_KEEPALIVE:
-				case SO_SNDBUF:
 					/* UNIMPL case SO_OOBINLINE: */
 					/* UNIMPL case SO_RCVBUF: */
 					/* UNIMPL case SO_SNDBUF: */
+					/* FAKE SO_SNDBUF, SO_RCVBUF, SO_TIMESTAMP */
+				case SO_RCVBUF:
+				case SO_SNDBUF:
+				case SO_TIMESTAMP:
 					/* UNIMPL case SO_RCVLOWAT: */
 					/* UNIMPL case SO_SNDLOWAT: */
 #if SO_REUSE
@@ -1673,6 +1799,7 @@ lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_t op
 					/* UNIMPL case IP_RCVIF: */
 				case IP_TTL:
 				case IP_TOS:
+					/* FAKE IP_MTU_DISCOVER */
 				case IP_MTU_DISCOVER:
 				case IP_RECVERR:
 				case IP_RECVTTL:
@@ -1694,7 +1821,7 @@ lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_t op
 				break;
 			}
 
-			/* If this is no TCP socket, ignore any options. */
+			/* If this is not a TCP socket, ignore any options. */
 			if ( sock->conn->type != NETCONN_TCP ) return 0;
 
 			switch( optname ) {
@@ -1809,9 +1936,11 @@ lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_t op
 #endif /* SO_REUSE */
 					/* UNIMPL case SO_USELOOPBACK: */
 					if ( *(int*)optval ) {
-						sock->conn->pcb.tcp->so_options |= so_map[optname];
+						//sock->conn->pcb.tcp->so_options |= so_map[optname];
+						pcb_access(sock->conn,OPT_SETBITS,OPT_SO_OPTIONS,so_map[optname]);
 					} else {
-						sock->conn->pcb.tcp->so_options &= ~(so_map[optname]);
+						//sock->conn->pcb.tcp->so_options &= ~(so_map[optname]);
+						pcb_access(sock->conn,OPT_CLRBITS,OPT_SO_OPTIONS,so_map[optname]);
 					}
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_setsockopt(%d, SOL_SOCKET, optname=0x%x, ..) -> %s\n", s, optname, (*(int*)optval?"on":"off")));
 					break;
@@ -1822,18 +1951,22 @@ lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_t op
 		case IPPROTO_IP:
 			switch( optname ) {
 				case IP_TTL:
-					sock->conn->pcb.tcp->ttl = (u8_t)(*(int*)optval);
+					//sock->conn->pcb.tcp->ttl = (u8_t)(*(int*)optval);
+					pcb_access(sock->conn,OPT_SETVALUE,OPT_TTL,(*(int*)optval));
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_setsockopt(%d, IPPROTO_IP, IP_TTL, ..) -> %u\n", s, sock->conn->pcb.tcp->ttl));
 					break;
 				case IP_TOS:
-					sock->conn->pcb.tcp->tos = (u8_t)(*(int*)optval);
+					//sock->conn->pcb.tcp->tos = (u8_t)(*(int*)optval);
+					pcb_access(sock->conn,OPT_SETVALUE,OPT_TOS,(*(int*)optval));
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_setsockopt(%d, IPPROTO_IP, IP_TOS, ..)-> %u\n", s, sock->conn->pcb.tcp->tos));
 					break;
 				case IP_HDRINCL:
 					if (*(int*)optval) 
-						sock->conn->pcb.tcp->so_options |= SOF_HDRINCL;
+						//sock->conn->pcb.tcp->so_options |= SOF_HDRINCL;
+						pcb_access(sock->conn,OPT_SETBITS,OPT_SO_OPTIONS,SOF_HDRINCL);
 					else
-						sock->conn->pcb.tcp->so_options &= ~SOF_HDRINCL;
+						//sock->conn->pcb.tcp->so_options &= ~SOF_HDRINCL;
+						pcb_access(sock->conn,OPT_CLRBITS,OPT_SO_OPTIONS,SOF_HDRINCL);
 					break;
 			}  /* switch */
 			break;
@@ -1843,14 +1976,17 @@ lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_t op
 			switch( optname ) {
 				case TCP_NODELAY:
 					if ( *(int*)optval ) {
-						sock->conn->pcb.tcp->flags |= TF_NODELAY;
+						//sock->conn->pcb.tcp->flags |= TF_NODELAY;
+						pcb_access(sock->conn,OPT_SETBITS,OPT_FLAGS,TF_NODELAY);
 					} else {
-						sock->conn->pcb.tcp->flags &= ~TF_NODELAY;
+						//sock->conn->pcb.tcp->flags &= ~TF_NODELAY;
+						pcb_access(sock->conn,OPT_CLRBITS,OPT_FLAGS,TF_NODELAY);
 					}
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_setsockopt(%d, IPPROTO_TCP, TCP_NODELAY) -> %s\n", s, (*(int *)optval)?"on":"off") );
 					break;
 				case TCP_KEEPALIVE:
-					sock->conn->pcb.tcp->keepalive = (u32_t)(*(int*)optval);
+					//sock->conn->pcb.tcp->keepalive = (u32_t)(*(int*)optval);
+					pcb_access(sock->conn,OPT_SETVALUE,OPT_KEEPALIVE,(u32_t)(*(int*)optval));
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_setsockopt(%d, IPPROTO_TCP, TCP_KEEPALIVE) -> %u\n", s, (int) sock->conn->pcb.tcp->keepalive));
 					break;
 			}  /* switch */
@@ -1861,7 +1997,8 @@ lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_t op
 				case IPV6_UNICAST_HOPS:
 				case IPV6_MULTICAST_HOPS:
 					/* TODO add a separate ttl for unicast */
-					sock->conn->pcb.tcp->ttl = (u8_t)(*(int*)optval);
+					//sock->conn->pcb.tcp->ttl = (u8_t)(*(int*)optval);
+					pcb_access(sock->conn,OPT_SETVALUE,OPT_TTL,(*(int*)optval));
 					LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_setsockopt(%d, IPPROTO_IPV6, IPV6_HOPLIMIT, ..) -> %u\n", s, sock->conn->pcb.tcp->ttl));
 					break;
 					/* TODO IPV6_HOPLIMIT is a flag to allow packet to inspect
@@ -1885,8 +2022,10 @@ lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_t op
 		case IPPROTO_RAW:
 			switch( optname ) {
 				case IPV6_CHECKSUM:
-					sock->conn->pcb.raw->so_options |= SOF_IPV6_CHECKSUM;
-					sock->conn->pcb.raw->checksumoffset=*(int*)optval;
+					//sock->conn->pcb.raw->so_options |= SOF_IPV6_CHECKSUM;
+					//sock->conn->pcb.raw->checksumoffset=*(int*)optval;
+					pcb_access(sock->conn,OPT_SETVALUE,OPT_CHECKSUMOFFSET,*(int*)optval);
+					break;
 				default:
 					break;
 			}
@@ -1898,14 +2037,13 @@ lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_t op
 	return err ? -1 : 0;
 }
 
-
 int multistack_cmd(int cmd, void *param);
 
-
-
 int lwip_ioctl(int s, unsigned long cmd, void *argp)
 {
 	struct lwip_socket *sock = get_socket(s);
+	struct stack *stack;
+	int cap;
 
 	if (!sock
 #if LWIP_NL
@@ -1915,7 +2053,7 @@ int lwip_ioctl(int s, unsigned long cmd, void *argp)
 			|| sock->family == PF_SOCKET
 #endif
 	   ) {
-		printf("lwip_ioctl %d %ld BADF\n",s,cmd);
+		LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_ioctl(%d, %u,... ) BADF\n", s, cmd));
 		set_errno(EBADF);
 		return -1;
 	}
@@ -1963,10 +2101,34 @@ int lwip_ioctl(int s, unsigned long cmd, void *argp)
 			}
 			return 0;
 		default:
-
+			stack=netconn_stack(sock->conn);
+#if LWIP_CAPABILITIES
+			if (stack->stack_capfun)
+				cap=stack->stack_capfun();
+			else
+				cap= ~0;
+#endif
 			if (cmd >= SIOCGIFNAME && cmd <= SIOCSIFTXQLEN) {
 				int err;
-				err=netif_ioctl(netconn_stack(sock->conn), cmd, argp);
+				err=netif_ioctl(stack, cmd, argp
+#if LWIP_CAPABILITIES
+						,cap
+#endif
+						);
+				sock_set_errno(sock, err);
+				LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_ioctl(%d, SIO TO NETIF) ret=%d\n",s,err));
+				if (err)
+					return -1;
+				else
+					return 0;
+			}
+			else if (cmd >= SIOCDARP && cmd <= SIOCSARP) {
+				int err=1;
+				err=etharp_ioctl(stack, cmd, argp
+#if LWIP_CAPABILITIES
+						,cap
+#endif
+						);
 				sock_set_errno(sock, err);
 				LWIP_DEBUGF(SOCKETS_DEBUG, ("lwip_ioctl(%d, SIO TO NETIF) ret=%d\n",s,err));
 				if (err)
@@ -1984,12 +2146,11 @@ int lwip_ioctl(int s, unsigned long cmd, void *argp)
 	}
 }
 
-
-int lwip_fcntl(int s, int cmd, long arg)
-{
-	return lwip_fcntl(s,cmd,arg);
-}
-
+#ifdef __USE_GNU
+#define FCNTL_SETFL_MASK (O_APPEND|O_ASYNC|O_DIRECT|O_NOATIME|O_NONBLOCK)
+#else
+#define FCNTL_SETFL_MASK (O_APPEND|O_ASYNC|O_NONBLOCK)
+#endif
 int lwip_fcntl64(int s, int cmd, long arg)
 {
 	struct lwip_socket *sock = get_socket(s);
@@ -2011,13 +2172,18 @@ int lwip_fcntl64(int s, int cmd, long arg)
 		case F_GETFL:
 			return sock->flags;
 		case F_SETFL:
-			sock->flags = arg;
+			sock->flags = (sock->flags & ~FCNTL_SETFL_MASK) | (arg & FCNTL_SETFL_MASK);
 			return 0;
 		default:
 			return -1;
 	}
 }
 
+int lwip_fcntl(int s, int cmd, long arg)
+{
+	return lwip_fcntl64(s,cmd,arg);
+}
+
 static int fdsplit(int max, 
 		fd_set *rfd,
 		fd_set *wfd,
@@ -2128,13 +2294,13 @@ int lwip_pselect(int maxfdp1, fd_set *readset, fd_set *writeset, fd_set *excepts
 			}
 		} else {
 			if (readset) {
-				if (FD_ISSET(i,readset)) count++; else FD_CLR(i,readset);
+				if (FD_ISSET(i,&lreadset)) count++; else FD_CLR(i,readset);
 			}
 			if (writeset) {
-				if (FD_ISSET(i,writeset)) count++; else FD_CLR(i,writeset);
+				if (FD_ISSET(i,&lwriteset)) count++; else FD_CLR(i,writeset);
 			}
 			if (exceptset) {
-				if (FD_ISSET(i,exceptset)) count++; else FD_CLR(i,exceptset);
+				if (FD_ISSET(i,&lexceptset)) count++; else FD_CLR(i,exceptset);
 			}
 		}
 	}
@@ -2292,13 +2458,13 @@ int lwip_poll(struct pollfd *fds, nfds_t nfds, int timeout) {
 }
 
 /* FIX: change implementations. Do not use a private buffer */
-int lwip_writev(int s, struct iovec *vector, int count)
+ssize_t lwip_writev(int s, struct iovec *vector, int count)
 {
-	int totsize=0;
+	ssize_t totsize=0;
 	int i;
-	int pos;
+	ssize_t pos;
 	char *temp_buf;
-	int ret;
+	ssize_t ret;
 
 	/* Check for invalid parameter */
 	if (count < 0 || count > UIO_MAXIOV) {
@@ -2338,13 +2504,13 @@ int lwip_writev(int s, struct iovec *vector, int count)
 	return ret;
 }
 
-int lwip_readv(int s, struct iovec *vector, int count)
+ssize_t lwip_readv(int s, struct iovec *vector, int count)
 {
-	int totsize=0;
+	ssize_t totsize=0;
 	int i;
-	int pos;
+	ssize_t pos;
 	char *temp_buf;
-	int ret;
+	ssize_t ret;
 
 	/* Check for invalid parameter */
 	if (count < 0 || count > UIO_MAXIOV) {
