@@ -2,7 +2,7 @@
  *   Developed for the Ale4NET project
  *   Application Level Environment for Networking
  *   
- *   Copyright 2004 Renzo Davoli University of Bologna - Italy
+ *   Copyright 2004,2011 Renzo Davoli University of Bologna - Italy
  *   
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -192,6 +192,7 @@ inet_chksum_pbuf(struct pbuf *p)
 
 
 /******************************************************************************/
+#if 0
 
 /* Here for now until needed in other places in lwIP */
 #ifndef isascii
@@ -356,7 +357,6 @@ char *inet_ntoa(struct in_addr addr)
 
 
 /******************************************************************************/
-
 /* FIX: review these */
 
 #include "lwip/ip_addr.h"
@@ -528,6 +528,7 @@ inet_pton(int af, const char *src, void *dst)
 #error BYTE_ORDER is not defined
 #endif
 
+#ifndef htons
 #if BYTE_ORDER == LITTLE_ENDIAN
 
 u16_t
@@ -556,8 +557,10 @@ ntohl(u32_t n)
 {
   return htonl(n);
 }
+#endif
 
 #endif /* BYTE_ORDER == LITTLE_ENDIAN */
+#endif
 
 
 
