@@ -57,7 +57,7 @@ static void ip_pmtu_free_list(struct pmtu_info  *head);
 
 /*---------------------------------------------------------------------------*/
 
-#ifdef ROUTE_DEBUG
+#if ROUTE_DEBUG == DBG_ON
 
 INLINE static void 
 sprintf_ip(char *str, struct ip_addr *addr)
@@ -107,7 +107,7 @@ void ip_route_debug_list(struct stack *stack)
 	}	
 }
 #else
-#define ip_route_debug_list() {}
+#define ip_route_debug_list(A)
 #endif
 
 /*---------------------------------------------------------------------------*/
@@ -887,23 +887,17 @@ ip_route_ipv6_select_source(struct netif *outif, struct ip_addr *dst)
 	return prefer;
 }
 
-
 struct ip_addr_list * 
 ip_route_select_source_ip(struct netif *outif, 
                        struct ip_addr *dest,
                        struct ip_addr *nexthop)
 {
-    if (ip_addr_is_v4comp(dest)) 
+	if (ip_addr_is_v4comp(dest)) 
 		return ip_addr_list_maskfind(outif->addrs, nexthop);
-    else 
+	else 
 		return ip_route_ipv6_select_source(outif, dest);
 }
 
-
-
-
-
-
 /* added by Diego Billi */
 #if 0
 
