@@ -8,7 +8,7 @@
  *   Developed for the Ale4NET project
  *   Application Level Environment for Networking
  *   
- *   Copyright 2004 Renzo Davoli University of Bologna - Italy
+ *   Copyright 2004,2011 Renzo Davoli University of Bologna - Italy
  *   
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -71,36 +71,217 @@
 #include "lwip/netif.h"
 #include "lwip/tcp.h"
 #include "lwip/stack.h"
+#include "lwip/memp.h"
 
 #ifndef NETIF_DEBUG
 #define NETIF_DEBUG DBG_OFF
 #endif
 
-void
+/* increase the number of fd available for interfaces of
+	 NETIF_MAX_STEP units when there are less than NETIF_MIN_FREE available */
+#define NETIF_MAX_STEP 8
+#define NETIF_MIN_FREE 4
+
+//#define NETIF_THREAD_DEBUG
+#ifdef NETIF_THREAD_DEBUG
+#define PRINTTHREAD(X) do {\
+	  printf("thread %s: %p\n",(X),pthread_self());\
+} while(0);
+#else
+#define PRINTTHREAD(X)
+#endif
+
+#define NETIF_UPDATE_FDLIST_WAKE 0
+#define NETIF_UPDATE_FDLIST_ADDFD 1
+struct netif_update_fdlist_data {
+	int op;
+	void *data;
+};
+
+struct netif_fddata *netif_addfd(struct netif *netif, int fd,
+		void (*fun)(struct netif_fddata *fddata, short revents),
+		void *opaque, int flags, short events) {
+	struct netif_fddata *new= memp_malloc(MEMP_NETIF_FDDATA);
+	if (new) {
+		struct netif_update_fdlist_data req = {
+			.op = NETIF_UPDATE_FDLIST_ADDFD,
+			.data = new
+		};
+		new->fd = fd;
+		new->netif = netif;
+		new->fun = fun;
+		new->opaque = opaque;
+		new->flags = flags;
+		new->events = events;
+		new->refcnt = 1;
+	  int rv=write(netif->stack->netif_pipe[1], &req, sizeof(req));
+		//fprintf(stderr,"netif_addfd request! %d\n",rv);
+	}
+	return new;
+}
+
+void netif_thread_wake(struct stack *stack)
+{
+	struct netif_update_fdlist_data req = {
+		.op = NETIF_UPDATE_FDLIST_WAKE, .data = NULL
+	};
+	write(stack->netif_pipe[1], &req, sizeof(req));
+}
+
+	static void
+netif_thread(void *arg)
+{
+	struct stack *stack=arg;
+	unsigned long time=time_now();
+	struct pollfd *pfd=mem_malloc(NETIF_MAX_STEP * sizeof(struct pollfd));
+	struct netif_fddata **pfddata=mem_malloc(NETIF_MAX_STEP * sizeof(struct netif_fddata *));
+	int pfdmax=NETIF_MAX_STEP;
+	int pfdlen=1;
+	int read_pipe_offset=0;
+	struct netif_update_fdlist_data read_pipe_item;
+	int i;
+
+	if (pfd == NULL || pfddata == NULL) {
+		fprintf(stderr,"netif_thread memory full: failed\n");
+		goto netif_thread_terminate;
+	}
+
+	pfd[0].fd = stack->netif_pipe[0];
+	pfd[0].events = POLLIN;
+	pfddata[0] = NULL;
+
+	PRINTTHREAD("NETIF_THREAD");
+	while(1) { /* stack active! This loop terminates when the pipe closes */
+		int ret;
+		unsigned long newtime;
+		/* eliminate closed files and copy events */
+		for (i=pfdlen-1; i>0; i--) {
+			if (pfddata[i] == NULL) {
+				//fprintf(stderr,"PFD #%d eliminated\n",i);
+				if (i < pfdlen-1) {
+					pfd[i] = pfd[pfdlen-1];
+					pfddata[i] = pfddata[pfdlen-1];
+				}
+				pfdlen--;
+			} else {
+				pfd[i].events = pfddata[i]->events;
+			}
+		}
+#if 0
+		for (i=0; i<pfdlen; i++)
+			fprintf(stderr, "%i-%d(%p,%d),",i,pfd[i].fd,pfddata[i],pfd[i].events);
+		fprintf(stderr, "netif POLL in %d\n",pfdlen);
+#endif
+		ret = poll(pfd, pfdlen, 100);
+#if 0
+		for (i=0; i<pfdlen; i++)
+			fprintf(stderr, "%i-%d(%p,%d),",i,pfd[i].fd,pfddata[i],pfd[i].revents);
+		fprintf(stderr, "netif POLL out %d\n",ret);
+#endif
+		if (pfd[0].revents) {
+			//fprintf(stderr, "netif GOT request\n");
+			if (pfd[0].revents & POLLNVAL || pfd[0].revents & POLLHUP)
+				break;
+			if (pfd[0].revents & POLLIN) {
+				int remaining = sizeof(struct netif_update_fdlist_data) - read_pipe_offset;
+				int n = read(stack->netif_pipe[0], 
+						(((u8_t *)&read_pipe_item)+read_pipe_offset), remaining);
+				if (n == 0)
+					break; /* pipe closed means thread termination <--------- */
+				if (n != remaining) {
+					read_pipe_offset += n;
+					continue;
+				}
+				//fprintf(stderr, "netif request isi %d\n", read_pipe_item.op);
+				switch (read_pipe_item.op) {
+					case NETIF_UPDATE_FDLIST_ADDFD:
+						{
+							if (pfdlen == pfdmax) {
+								int newmax = pfdmax + NETIF_MAX_STEP;
+								struct pollfd *newpfd = mem_realloc(pfd, newmax * sizeof(struct pollfd)); 
+								struct netif_fddata **newpfddata=mem_realloc(pfddata, newmax * sizeof(struct netif_fddata *));
+								if (newpfd) pfd = newpfd;
+								if (newpfddata) pfddata = newpfddata;
+								if (newpfd && newpfddata)
+									pfdmax = newmax;
+							}
+							if (pfdlen < pfdmax) {
+								struct netif_fddata *data = pfddata[pfdlen] = read_pipe_item.data;
+								pfd[pfdlen].fd = data->fd;
+								pfd[pfdlen].events = data->events;
+								pfdlen++;
+							}
+						}
+				}
+			}
+			continue;
+		}
+
+		for (i=1; ret>0 && i<pfdlen; i++) {
+			if (pfd[i].revents != 0) {
+				ret--;
+				if (pfd[i].revents & POLLNVAL /*|| pfd[i].revents & POLLHUP*/) { /* closed! delete the element */
+					//fprintf(stderr,"netif fd %d was closed\n",pfd[i].fd);
+					memp_free(MEMP_NETIF_FDDATA,pfddata[i]);
+					pfddata[i] = NULL;
+					pfd[i].fd = -1;
+				} else {
+					//fprintf(stderr, "netif request CALL %p %d %d\n", pfddata[i]->fun, pfd[i].fd, pfd[i].revents);
+					pfddata[i]->fun(pfddata[i], pfd[i].revents);
+				}
+			}
+		}
+		newtime=time_now();
+		if (newtime > time) {
+			time=newtime;
+			for (i=1; i<pfdlen; i++)
+				if (pfddata[i] && pfddata[i]->flags & NETIF_ARGS_1SEC_POLL) 
+					pfddata[i]->fun(pfddata[i], 0);
+		}
+	}
+netif_thread_terminate:
+	for (i=1; i<pfdlen; i++)
+		mem_free(pfddata[i]);
+	if (pfd) mem_free(pfd);
+	if (pfddata) mem_free(pfddata);
+	LWIP_DEBUGF( NETIF_DEBUG, ("netif_thread leaving loop \n"));
+
+	sys_sem_signal(stack->netif_cleanup_mutex);
+}
+
+	void
 netif_init(struct stack *stack)
 {
 	/* FIX: move ip_addr_list_init() to ip6.c? */
 
-	//ip_addr_list_init();
 	ip_addr_list_init(stack);
 
 	//ip_route_list_init(stack); 
-	
+
 	stack->netif_list = NULL;
+
+	/* add some fds for interfaces */
+
+	pipe(stack->netif_pipe);
+	stack->netif_cleanup_mutex = sys_sem_new(0);
+
+	sys_thread_new(netif_thread, stack, DEFAULT_THREAD_PRIO);
 }
 
-void
+	void
 netif_shutdown(struct stack *stack)
 {
   netif_cleanup(stack);
   
-  /* FIX: TODO */
+  LWIP_DEBUGF( NETIF_DEBUG, ("netif_shutdown!\n") );
+	close(stack->netif_pipe[1]);
 
+	sys_sem_wait_timeout(stack->netif_cleanup_mutex, 0);
+	sys_sem_free(stack->netif_cleanup_mutex);
+	close(stack->netif_pipe[0]);
   LWIP_DEBUGF( NETIF_DEBUG, ("netif_shutdown: done!\n") );
 }
 
-
-
 /**
  * Add a network interface to the list of lwIP netifs.
  *
@@ -144,12 +325,12 @@ struct netif * netif_add(
   netif->addrs = NULL;
 
   netif->input = input;
-  netif->cleanup = NULL;
+  netif->netifctl = NULL;
   netif->change = change;
 
   netif->id = ++stack->uniqueid;
 
-  netif->flags = NETIF_RUNNING;
+  netif->flags |= (NETIF_FLAG_LINK_UP | IFF_RUNNING);
   /* printf("netif_add %x netif->input %x\n",netif,netif->input); */
 
   /* call user specified initialization function for netif */
@@ -159,11 +340,17 @@ struct netif * netif_add(
   }
 
 #if IPv6_AUTO_CONFIGURATION  
-  ip_autoconf_netif_init(netif);
+	if (netif->flags & NETIF_FLAG_AUTOCONF)
+		ip_autoconf_netif_init(netif);
+	else
+		netif->autoconf = NULL;
 #endif
 
 #if IPv6_ROUTER_ADVERTISEMENT
-  ip_radv_netif_init(netif);
+	if (netif->flags & NETIF_FLAG_RADV)
+		ip_radv_netif_init(netif);
+	else
+		netif->radv = NULL;
 #endif
 
   if (stack->netif_list == NULL)
@@ -316,8 +503,8 @@ void netif_remove(struct netif * netif)
   
   ip_route_list_delnetif(stack, netif);
   
-  if(netif->cleanup)
-    netif->cleanup(netif);
+  if(netif->netifctl)
+    netif->netifctl(netif,NETIFCTL_CLEANUP,NULL);
     
   LWIP_DEBUGF( NETIF_DEBUG, ("netif_remove: removed netif\n") );
 }
@@ -376,10 +563,14 @@ netif_cleanup(struct stack *stack)
 {
 	struct netif *nip;
 	
-	for (nip=stack->netif_list; nip!=NULL; nip=nip->next)
-		// FIX: shutdown interface? RA needs this.
-		if (nip->cleanup)
-			nip->cleanup(nip);
+	for (nip=stack->netif_list; nip!=NULL; nip=nip->next) {
+		// shutdown interface
+		if ((nip->flags & IFF_UP) && (nip->change))
+			nip->change(nip, NETIF_CHANGE_DOWN);
+
+		if (nip->netifctl)
+			nip->netifctl(nip,NETIFCTL_CLEANUP,NULL);
+	}
 }
 
 
@@ -392,7 +583,7 @@ static int netif_ifconf(struct stack *stack, struct ifconf *ifc)
 	//printf("%s\n", __func__);
 #define ifr_v (ifc->ifc_req)
 
-	/*printf("-netif_ifconf %d\n",ifc->ifc_len);*/
+	/*printf("-netif_ifconf %p %p %d\n",ifc,ifc->ifc_req,ifc->ifc_len);*/
 	ifc->ifc_len=0;
 	memset(ifr_v, 0, maxlen); /* jjsa clear the memory area */
 	for (nip=stack->netif_list, i=0; nip!=NULL && ifc->ifc_len < maxlen; 
@@ -429,7 +620,7 @@ static int netif_ifconf(struct stack *stack, struct ifconf *ifc)
 		}
 	}
 	/*{int i;
-		printf("len %d %d\n",ifc->ifc_len,sizeof(struct ifreq));
+		printf("%p len %d %d\n",ifc->ifc_req,ifc->ifc_len,sizeof(struct ifreq));
 		for (i=0;i<ifc->ifc_len;i++) {
 		int c=*(((unsigned char *)(ifr_v))+i);
 		printf("%02x%c ",c,(c>=' '&&c<='~')?c:'.');
@@ -444,18 +635,60 @@ static int netif_ifconf(struct stack *stack, struct ifconf *ifc)
 #undef ifr_v
 }
 
-int netif_ioctl(struct stack *stack, int cmd,struct ifreq *ifr)
+int netif_ioctl(struct stack *stack, int cmd,struct ifreq *ifr
+#if LWIP_CAPABILITIES
+		,int cap
+#endif
+		)
 {
 	u16_t oldflags;
 	int retval;
 	struct netif *nip;
 	register int i;
 
+	/*printf("netif_ioctl %x %p\n",cmd,ifr);*/
 	if (ifr == NULL)
 		retval=EFAULT;
 	else {
+#if LWIP_CAPABILITIES
+		switch (cmd) {
+			case SIOCSIFLINK:
+			case SIOCSIFFLAGS:
+			case SIOCSIFADDR:
+			case SIOCSIFDSTADDR:
+			case SIOCSIFBRDADDR:
+			case SIOCSIFNETMASK:
+			case SIOCSIFMETRIC:
+			case SIOCSIFMEM:
+			case SIOCSIFMTU:
+			case SIOCSIFNAME:
+			case SIOCSIFHWADDR:
+			case SIOCSIFENCAP:
+			case SIOCSIFSLAVE:
+			case SIOCADDMULTI:
+			case SIOCDELMULTI:
+			case SIOCSIFPFLAGS:
+			case SIOCSIFHWBROADCAST:
+			case SIOCSIFBR:
+			case SIOCSIFTXQLEN:
+				if ((cap&LWIP_CAP_NET_ADMIN) == 0)
+					return EPERM;
+		}
+#endif
 		if (cmd == SIOCGIFCONF) {
 			retval=netif_ifconf(stack, (struct ifconf *)ifr);
+		} else if (cmd == SIOCGIFNAME) {
+			if ((nip = netif_find_id(stack, ifr->ifr_ifindex)) == NULL)
+				 retval=EINVAL;
+			else {
+				ifr->ifr_name[0]=nip->name[0];
+				ifr->ifr_name[1]=nip->name[1];
+				ifr->ifr_name[2]=(nip->num%10)+'0';
+				ifr->ifr_name[3]= 0;
+				ifr->ifr_name[4]= 0;
+				ifr->ifr_name[5]= 0;
+				retval=ERR_OK;
+			}
 		} else {
 #define ifrname ifr->ifr_name
 			ifrname[4]=ifrname[5]=0;
@@ -495,11 +728,10 @@ int netif_ioctl(struct stack *stack, int cmd,struct ifreq *ifr)
 															LWIP_DEBUGF( NETIF_DEBUG, ("SIOCGIFADDR\n"));
 														}
 														retval = ERR_OK; break;
-														//retval = ENOSYS; break;
 
 					case SIOCGIFFLAGS: 
 						LWIP_DEBUGF( NETIF_DEBUG, ("SIOCGIFFLAGS %x\n",nip->flags));
-						ifr->ifr_flags= nip->flags & ~(IFF_RUNNING);
+						ifr->ifr_flags= nip->flags & ~NETIF_IFF_INCOMPATIBLE_MASK;
 						retval=ERR_OK; 
 						break;
 					case SIOCSIFFLAGS:
@@ -511,7 +743,7 @@ int netif_ioctl(struct stack *stack, int cmd,struct ifreq *ifr)
 							if (nip->change)
 							nip->change(nip, NETIF_CHANGE_DOWN);
 
-						nip->flags = (nip->flags & IFF_RUNNING) | (ifr->ifr_flags & ~(IFF_RUNNING));
+						nip->flags = (nip->flags & NETIF_IFF_INCOMPATIBLE_MASK) | (ifr->ifr_flags & ~NETIF_IFF_INCOMPATIBLE_MASK);
 
 						/* If interface is now up */
 						if ( !(oldflags & IFF_UP) &&  (ifr->ifr_flags & IFF_UP) )
@@ -532,10 +764,19 @@ int netif_ioctl(struct stack *stack, int cmd,struct ifreq *ifr)
 
 					case SIOCGIFHWADDR:
 						ifr->ifr_hwaddr.sa_family=nip->type;
+						if (nip->hwaddr_len > IFHWADDRLEN)
+							nip->hwaddr_len = IFHWADDRLEN;
 						for (i=0;i<nip->hwaddr_len;i++)
 						ifr->ifr_hwaddr.sa_data[i]=nip->hwaddr[i];
 						retval=ERR_OK; break;
 
+					case SIOCSIFHWADDR:
+						ifr->ifr_hwaddr.sa_family==nip->type;
+						nip->hwaddr_len = IFHWADDRLEN;
+						for (i=0;i<nip->hwaddr_len;i++)
+							nip->hwaddr[i]=ifr->ifr_hwaddr.sa_data[i];
+						retval=ERR_OK; break;
+
 					case SIOCGIFINDEX:
 						ifr->ifr_ifindex=nip->id;
 						retval=ERR_OK; break;
@@ -559,9 +800,9 @@ u8_t netif_is_up(struct netif *netif)
 	return (netif->flags & NETIF_FLAG_UP)?1:0;
 }
 
-void netif_set_up(struct netif *netif)
+void netif_set_up(struct netif *netif, int flags)
 {
-	netif->flags |= NETIF_FLAG_UP;
+	netif->flags |= (NETIF_FLAG_UP | (flags & NETIF_IFUP_FLAGS));
 	
 	if (netif->change)
 		netif->change(netif, NETIF_CHANGE_UP);
@@ -585,8 +826,6 @@ void netif_set_down_low(struct netif *netif)
 	netif->flags &= ~NETIF_FLAG_UP;
 }
 
-
-
 #if LWIP_NL
 
 static void netif_out_link_address (int index,struct netif *nip,void * buf,int *offset) {
@@ -667,7 +906,7 @@ static void netif_netlink_link_out(struct nlmsghdr *msg,struct netif *nip,void *
 	ifi.__ifi_pad=0;
 	ifi.ifi_type=nip->type; 
 	ifi.ifi_index=nip->id;
-	ifi.ifi_flags=nip->flags; 
+	ifi.ifi_flags=nip->flags & ~NETIF_IFF_INCOMPATIBLE_MASK; 
 	ifi.ifi_change=0xffffffff;
 	netlink_addanswer(buf,offset,&ifi,sizeof (struct ifinfomsg));
 
@@ -797,7 +1036,6 @@ void netif_netlink_getaddr(struct stack *stack, struct nlmsghdr *msg,void * buf,
 		if ((flag & NLM_F_DUMP) == NLM_F_DUMP ||
 				ifa->ifa_index == nip->id) {
 			struct ip_addr_list *ial=nip->addrs;
-			/*for(ial=nip->addrs;ial != NULL;ial=ial->next)*/
 			if (ial != NULL) {
 				ial=nip->addrs->next;
 				do {
@@ -808,7 +1046,6 @@ void netif_netlink_getaddr(struct stack *stack, struct nlmsghdr *msg,void * buf,
 					ial=ial->next;
 				} while (ial != nip->addrs->next);
 			}
-
 		}
 	}
 	msg->nlmsg_type = NLMSG_DONE;
