@@ -30,6 +30,27 @@
  * loop end condition (tot_len == p->len), NOT (next == NULL).
  */
 
+/*   This is part of LWIPv6
+ *   Developed for the Ale4NET project
+ *   Application Level Environment for Networking
+ *   
+ *   Copyright 2011 Renzo Davoli University of Bologna - Italy
+ *   
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, write to the Free Software Foundation, Inc.,
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ */
+
 /*
  * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
  * All rights reserved.
@@ -73,7 +94,6 @@
 #include "lwip/sys.h"
 #include "arch/perf.h"
 
-//#ifdef LWIP_NAT
 #if LWIP_USERFILTER && LWIP_NAT
 #include "lwip/netif.h"
 #include "lwip/ip.h"
@@ -195,7 +215,6 @@ pbuf_alloc(pbuf_layer l, u16_t length, pbuf_flag flag)
   }
 
 /* added by Diego Billi */
-//#ifdef LWIP_NAT
 #if LWIP_USERFILTER && LWIP_NAT
   nat_pbuf_init(p);
 #endif
@@ -214,27 +233,6 @@ pbuf_alloc(pbuf_layer l, u16_t length, pbuf_flag flag)
 #define DEC_PBUF_STATS
 #endif /* PBUF_STATS */
 
-#define PBUF_POOL_FAST_FREE(p)  do {                                    \
-                                  p->next = pbuf_pool;                  \
-                                  pbuf_pool = p;                        \
-                                  DEC_PBUF_STATS;                       \
-                                } while (0)
-
-#if SYS_LIGHTWEIGHT_PROT
-#define PBUF_POOL_FREE(p)  do {                                         \
-                                SYS_ARCH_DECL_PROTECT(old_level);       \
-                                SYS_ARCH_PROTECT(old_level);            \
-                                PBUF_POOL_FAST_FREE(p);                 \
-                                SYS_ARCH_UNPROTECT(old_level);          \
-                               } while (0)
-#else /* SYS_LIGHTWEIGHT_PROT */
-#define PBUF_POOL_FREE(p)  do {                                         \
-                             sys_sem_wait(pbuf_pool_free_sem);          \
-                             PBUF_POOL_FAST_FREE(p);                    \
-                             sys_sem_signal(pbuf_pool_free_sem);        \
-                           } while (0)
-#endif /* SYS_LIGHTWEIGHT_PROT */
-
 /**
  * Shrink a pbuf chain to a desired length.
  *
@@ -445,7 +443,6 @@ pbuf_free(struct pbuf *p)
       q = p->next;
 
 /* added by Diego Billi */
-//#ifdef LWIP_NAT
 #if LWIP_USERFILTER && LWIP_NAT
       nat_pbuf_put(p);
 #endif
@@ -678,13 +675,13 @@ pbuf_dequeue(struct pbuf *p)
 
 /**
  *
- * Create PBUF_POOL (or PBUF_RAM) copies of PBUF_REF pbufs.
+ * Create PBUF_RAM copies of PBUF_REF pbufs.
  *
  * Used to queue packets on behalf of the lwIP stack, such as
  * ARP based queueing.
  *
  * Go through a pbuf chain and replace any PBUF_REF buffers
- * with PBUF_POOL (or PBUF_RAM) pbufs, each taking a copy of
+ * with PBUF_RAM pbufs, each taking a copy of
  * the referenced data.
  *
  * @note You MUST explicitly use p = pbuf_take(p);
@@ -714,7 +711,6 @@ pbuf_take(struct pbuf *p)
     if (p->flags == PBUF_FLAG_REF) {
       LWIP_DEBUGF(PBUF_DEBUG | DBG_TRACE, ("pbuf_take: encountered PBUF_REF %p\n", (void *)p));
       /* allocate a pbuf (w/ payload) fully in RAM */
-      /* PBUF_POOL buffers are faster if we can use them */
         q = pbuf_alloc(PBUF_RAW, p->len, PBUF_RAM);
         if (q == NULL) {
           LWIP_DEBUGF(PBUF_DEBUG | DBG_TRACE | 2, ("pbuf_take: Could not allocate PBUF_RAM\n"));
@@ -811,9 +807,6 @@ pbuf_dechain(struct pbuf *p)
   return (tail_gone > 0? NULL: q);
 }
 
-
-
-
 /* added by Diego Billi */
 struct pbuf * pbuf_clone(pbuf_layer l, struct pbuf *p, pbuf_flag flag)
 {
