@@ -8,7 +8,7 @@
  *   Developed for the Ale4NET project
  *   Application Level Environment for Networking
  *   
- *   Copyright 2004 Renzo Davoli University of Bologna - Italy
+ *   Copyright 2004,2010,2011 Renzo Davoli University of Bologna - Italy
  *   
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -97,8 +97,6 @@ struct tcp_pcb *tcp_active_pcb; /* List of all TCP PCBs that are in a
 	 
 struct tcp_pcb *tcp_tw_pcbs;      /* List of all TCP PCBs in TIME-WAIT. */
 
-struct tcp_pcb *tcp_tmp_pcb;
-
 static u8_t tcp_timer;
 
 #endif
@@ -119,7 +117,6 @@ tcp_init(struct stack *stack)
   stack->tcp_listen_pcbs.listen_pcbs = NULL;
   stack->tcp_active_pcbs             = NULL;
   stack->tcp_tw_pcbs                 = NULL;
-  stack->tcp_tmp_pcb                 = NULL;
   
   /* initialize timer */
   stack->tcp_ticks = 0;
@@ -166,6 +163,7 @@ tcp_close(struct tcp_pcb *pcb)
   
   err_t err;
 
+
 #if TCP_DEBUG
   LWIP_DEBUGF(TCP_DEBUG, ("tcp_close: closing in state "));
   tcp_debug_print_state(pcb->state);
@@ -467,9 +465,14 @@ tcp_listen(struct tcp_pcb *pcb)
   }
   
   lpcb->stack = stack;
+#ifdef LWSLIRP
+	lpcb->slirp_fddata = pcb->slirp_fddata;
+	lpcb->slirp_state = pcb->slirp_state;
+#endif
   
   lpcb->callback_arg = pcb->callback_arg;
   lpcb->local_port = pcb->local_port;
+  lpcb->remote_port = pcb->remote_port;
   lpcb->state = LISTEN;
   lpcb->so_options = pcb->so_options;
   lpcb->so_options |= SOF_ACCEPTCONN;
@@ -1007,6 +1010,10 @@ tcp_alloc(struct stack *stack, u8_t prio)
     memset(pcb, 0, sizeof(struct tcp_pcb));
     
     pcb->stack = stack;
+#ifdef LWSLIRP
+		pcb->slirp_fddata = NULL;
+		pcb->slirp_state = SS_NOFDREF;
+#endif
     
     pcb->prio = TCP_PRIO_NORMAL;
     pcb->snd_buf = TCP_SND_BUF;
@@ -1066,6 +1073,7 @@ tcp_new(struct stack *stack)
 void
 tcp_arg(struct tcp_pcb *pcb, void *arg)
 {  
+	//fprintf(stderr,"tcp_arg %p %p\n",pcb,arg);
   pcb->callback_arg = arg;
 }
 #if LWIP_CALLBACK_API
@@ -1082,6 +1090,7 @@ void
 tcp_recv(struct tcp_pcb *pcb,
    err_t (* recv)(void *arg, struct tcp_pcb *tpcb, struct pbuf *p, err_t err))
 {
+	//fprintf(stderr,"tcp_recv %p %p\n",pcb,recv);
   pcb->recv = recv;
 }
 
@@ -1097,6 +1106,7 @@ void
 tcp_sent(struct tcp_pcb *pcb,
    err_t (* sent)(void *arg, struct tcp_pcb *tpcb, u16_t len))
 {
+	//fprintf(stderr,"tcp_sent %p %p\n",pcb,sent);
   pcb->sent = sent;
 }
 
@@ -1112,6 +1122,7 @@ void
 tcp_err(struct tcp_pcb *pcb,
    void (* errf)(void *arg, err_t err))
 {
+	//fprintf(stderr,"tcp_err %p %p\n",pcb,errf);
   pcb->errf = errf;
 }
 
@@ -1127,6 +1138,7 @@ void
 tcp_accept(struct tcp_pcb *pcb,
      err_t (* accept)(void *arg, struct tcp_pcb *newpcb, err_t err))
 {
+	//fprintf(stderr,"tcp_accept %p %p\n",pcb,accept);
   ((struct tcp_pcb_listen *)pcb)->accept = accept;
 }
 #endif /* LWIP_CALLBACK_API */
@@ -1165,6 +1177,11 @@ tcp_pcb_purge(struct tcp_pcb *pcb)
      pcb->state != TIME_WAIT &&
      pcb->state != LISTEN) {
 
+#ifdef LWSLIRP
+		if (pcb->slirp_fddata != NULL)
+			slirp_tcp_close(pcb);
+#endif
+
     LWIP_DEBUGF(TCP_DEBUG, ("tcp_pcb_purge\n"));
     
     if (pcb->unsent != NULL) {    
@@ -1335,6 +1352,7 @@ void
 tcp_debug_print_pcbs(void)
 {
   struct tcp_pcb *pcb;
+  struct stack *stack=pcb->stack;
   LWIP_DEBUGF(TCP_DEBUG, ("Active PCB states:\n"));
   for(pcb = stack->tcp_active_pcbs; pcb != NULL; pcb = pcb->next) {
     LWIP_DEBUGF(TCP_DEBUG, ("Local port %u, foreign port %u snd_nxt %lu rcv_nxt %lu ",
@@ -1362,6 +1380,7 @@ int
 tcp_pcbs_sane(void)
 {
   struct tcp_pcb *pcb;
+  struct stack *stack=pcb->stack;
   for(pcb = stack->tcp_active_pcbs; pcb != NULL; pcb = pcb->next) {
     LWIP_ASSERT("tcp_pcbs_sane: active pcb->state != CLOSED", pcb->state != CLOSED);
     LWIP_ASSERT("tcp_pcbs_sane: active pcb->state != LISTEN", pcb->state != LISTEN);
