@@ -8,7 +8,7 @@
  *   Developed for the Ale4NET project
  *   Application Level Environment for Networking
  *
- *   Copyright 2004 Renzo Davoli University of Bologna - Italy
+ *   Copyright 2004,2010,2011 Renzo Davoli University of Bologna - Italy
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -514,18 +514,21 @@ tcp_output(struct tcp_pcb *pcb)
         useg = seg;
       /* unacked list is not empty? */
       } else {
-        /* In the case of fast retransmit, the packet should not go to the tail
-         * of the unacked queue, but rather at the head. We need to check for
-         * this case. -STJ Jul 27, 2004 */
-        if (TCP_SEQ_LT(ntohl(seg->tcphdr->seqno), ntohl(useg->tcphdr->seqno))){
-          /* add segment to head of unacked list */
-          seg->next = pcb->unacked;
-          pcb->unacked = seg;
-        } else {
-          /* add segment to tail of unacked list */
-          useg->next = seg;
-          useg = useg->next;
-        }
+				/********** XXX XXX *********/
+				if (useg == NULL)
+					printf("??????? race condition error ??? seg useg %p %p\n",seg,useg);
+				/* In the case of fast retransmit, the packet should not go to the tail
+				 * of the unacked queue, but rather at the head. We need to check for
+				 * this case. -STJ Jul 27, 2004 */
+				if (TCP_SEQ_LT(ntohl(seg->tcphdr->seqno), ntohl(useg->tcphdr->seqno))){
+					/* add segment to head of unacked list */
+					seg->next = pcb->unacked;
+					pcb->unacked = seg;
+				} else {
+					/* add segment to tail of unacked list */
+					useg->next = seg;
+					useg = useg->next;
+				}
       }
     /* do not queue empty segments on the unacked list */
     } else {
@@ -619,6 +622,14 @@ tcp_rst(struct stack *stack, u32_t seqno, u32_t ackno,
       return;
   }
 
+#if 0
+	fprintf(stderr, "RESET! %x %x %x %x:%d %x %x %x %x %d\n",
+			local_ip->addr[0], local_ip->addr[1], local_ip->addr[2], local_ip->addr[3],
+			local_port,
+			remote_ip->addr[0], remote_ip->addr[1], remote_ip->addr[2], remote_ip->addr[3],
+			remote_port);
+#endif
+
   tcphdr = p->payload;
   tcphdr->src = htons(local_port);
   tcphdr->dest = htons(remote_port);
