@@ -39,7 +39,6 @@
 #include "lwip/raw.h"
 #include "lwip/tcp.h"
 #include "lwip/api.h"
-#include "lwip/api_msg.h"
 #include "lwip/tcpip.h"
 
 
@@ -53,14 +52,17 @@ typedef enum {
 
   MEMP_NETBUF,
   MEMP_NETCONN,
-  MEMP_API_MSG,
   MEMP_TCPIP_MSG,
 
   MEMP_SYS_TIMEOUT,
 
 	MEMP_ROUTE,
 	MEMP_ADDR,
+	MEMP_NETIF_FDDATA,
+
+#if IPv4_FRAGMENTATION || IPv6_FRAGMENTATION
 	MEMP_REASS,
+#endif
 
 /* added by Diego Billi */  
 #if LWIP_USERFILTER && LWIP_NAT
@@ -71,19 +73,31 @@ typedef enum {
   MEMP_MAX
 } memp_t;
 
+#ifndef DEBUGMEM
 void memp_init(void);
 
 void *memp_malloc(memp_t type);
-void *memp_realloc(memp_t fromtype, memp_t totype, void *mem);
 void memp_free(memp_t type, void *mem);
+#else
+void memp_d_init(char *file, int line);
 
-#endif /* __LWIP_MEMP_H__  */
+void *memp_d_malloc(memp_t type, char *file, int line);
+void memp_d_free(memp_t type, void *mem, char *file, int line);
+
+#define memp_init() memp_d_init(__FILE__,__LINE__)
+#define memp_malloc(X) memp_d_malloc((X),__FILE__,__LINE__)
+#define memp_free(X,Y) memp_d_free((X), (Y), __FILE__,__LINE__)
+#endif
 
 
-/*
+
+#if 0
 #define memp_init() ({ ; })
 #define memp_free(T,X) ({ printf("MEMP-FREE %x %s %d\n",(X),__FILE__,__LINE__); \
 		    free(X); })
+#if LWIP_USERFILTER && LWIP_NAT
+#include "lwip/nat/nat.h"
+
 #define memp_malloc(T) ({ void *x; \
 		u16_t memp_sizes[MEMP_MAX] = {\
 		sizeof(struct pbuf),\
@@ -94,14 +108,18 @@ void memp_free(memp_t type, void *mem);
 		sizeof(struct tcp_seg),\
 		sizeof(struct netbuf),\
 		sizeof(struct netconn),\
-		sizeof(struct api_msg),\
 		sizeof(struct tcpip_msg),\
-		sizeof(struct sys_timeout)\
+		sizeof(struct sys_timeout),\
+		sizeof(struct ip_addr_list),\
+		sizeof(struct ip_reassbuf),\
+		sizeof(struct nat_pcb),\
+		sizeof(struct nat_rule)\
 		};\
 		x=malloc(memp_sizes[T]); \
 		printf("MEMP-MALLOC %x (T=%d Size=%d) %s %d\n",x,T,memp_sizes[T],__FILE__,__LINE__); \
 		x; })
-#define memp_realloc(OLDT,NEWT,X) ({ void *x,*old; \
+#else
+#define memp_malloc(T) ({ void *x; \
 		u16_t memp_sizes[MEMP_MAX] = {\
 		sizeof(struct pbuf),\
 		sizeof(struct raw_pcb),\
@@ -111,13 +129,15 @@ void memp_free(memp_t type, void *mem);
 		sizeof(struct tcp_seg),\
 		sizeof(struct netbuf),\
 		sizeof(struct netconn),\
-		sizeof(struct api_msg),\
 		sizeof(struct tcpip_msg),\
-		sizeof(struct sys_timeout)\
+		sizeof(struct sys_timeout),\
+		sizeof(struct ip_addr_list),\
+		sizeof(struct ip_reassbuf)\
 		};\
-		old=(X);\
-		x=realloc(old,memp_sizes[NEWT]); \
-		printf("MEMP-REALLOC %x->%x (T=%d Size=%d) %s %d\n",old,x,T,memp_sizes[T],__FILE__,__LINE__); \
+		x=malloc(memp_sizes[T]); \
+		printf("MEMP-MALLOC %x (T=%d Size=%d) %s %d\n",x,T,memp_sizes[T],__FILE__,__LINE__); \
 		x; })
-*/
-    
+#endif
+#endif
+#endif /* __LWIP_MEMP_H__  */
+
