@@ -32,9 +32,11 @@
 #ifndef __LWIP_TCPIP_H__
 #define __LWIP_TCPIP_H__
 
-//#include "lwip/api_msg.h"
 #include "lwip/pbuf.h"
 #include "lwip/stack.h"
+#if LWIP_CAPABILITIES
+typedef int (*lwip_capfun)(void);
+#endif
 
 enum tcpip_msg_type {
 
@@ -42,6 +44,7 @@ enum tcpip_msg_type {
   TCPIP_MSG_API,
   TCPIP_MSG_INPUT,
   TCPIP_MSG_CALLBACK,
+  TCPIP_MSG_SYNC_CALLBACK,
 
   /* other messages */
   TCPIP_MSG_NETIFADD,
@@ -49,6 +52,11 @@ enum tcpip_msg_type {
   TCPIP_MSG_NETIF_NOTIFY
 };
 
+enum tcpip_sync {
+	ASYNC = 0,
+	SYNC = 1
+};
+
 struct tcpip_msg {
 
   enum tcpip_msg_type type;
@@ -64,6 +72,7 @@ struct tcpip_msg {
     } inp;
 
     struct {
+      sys_sem_t *sem;    // used for synchronous calls
       void (*f)(void *ctx);
       void *ctx;
     } cb;
@@ -71,7 +80,7 @@ struct tcpip_msg {
 
     /* Signal to the main thread to add a new network interface */
     struct {
-      sys_sem_t *sem;    // used for syncronous calls
+      sys_sem_t *sem;    // used for synchronous calls
       struct netif *netif;
       void *state;
       err_t (* init)(struct netif *netif);
@@ -82,7 +91,7 @@ struct tcpip_msg {
 
 
     struct {
-      sys_sem_t *sem;    // used for syncronous calls
+      sys_sem_t *sem;    // used for synchronous calls
       struct netif *netif;
       u32_t type;
     } netif_notify;
@@ -95,8 +104,12 @@ int tcpip_init(void);
 
 typedef void (* tcpip_handler)(void *arg);
 
+#if LWIP_CAPABILITIES
+struct stack *tcpip_start(tcpip_handler init_func, void *arg, unsigned long flags, lwip_capfun capfun);
+#else
 /* Alloc a new stack thread and return stack number */
-struct stack *tcpip_start(tcpip_handler init_func, void *arg);
+struct stack *tcpip_start(tcpip_handler init_func, void *arg, unsigned long flags);
+#endif
 
 /* Signal to the stack to shutdown */
 void tcpip_shutdown(struct stack *stack, tcpip_handler shutdown_func, void *arg);
@@ -112,7 +125,7 @@ struct stack *tcpip_stack_set(struct stack *id);
    After tcpip_shutdown() they are unuseful. */
 void  tcpip_apimsg(struct stack *stack, struct api_msg *apimsg);
 err_t tcpip_input(struct pbuf *p, struct netif *inp);
-err_t tcpip_callback(struct stack *stack, void (*f)(void *ctx), void *ctx);
+err_t tcpip_callback(struct stack *stack, void (*f)(void *ctx), void *ctx, enum tcpip_sync sync);
 
 void tcpip_tcp_timer_needed(struct stack *stack);
 
