@@ -43,6 +43,7 @@
 #include "lwip/ip_addr.h"
 #include "lwip/netif.h"
 #include "lwip/ip.h"
+#include "lwip/sockets.h"
 
 #ifdef PACK_STRUCT_USE_INCLUDES
 #  include "arch/bpstruct.h"
@@ -167,5 +168,13 @@ err_t etharp_output(struct netif *netif, struct ip_addr *ipaddr,
          struct pbuf *q);
 err_t etharp_query(struct ip_addr_list *al, struct ip_addr *ipaddr, struct pbuf *q);
 err_t etharp_request(struct ip_addr_list *al, struct ip_addr *ipaddr);
+int etharp_ioctl(struct stack *stack, int cmd,struct arpreq *ifr
+#if LWIP_CAPABILITIES
+		,int cap
+#endif
+		);
+err_t update_arp_entry(struct netif *netif, struct ip_addr *ipaddr, struct eth_addr *ethaddr, u32_t flags);
+
+
 
 #endif /* __NETIF_ARP_H__ */
