@@ -28,6 +28,7 @@
 #include "lwip/debug.h"
 #include "lwip/sys.h"
 #include "lwip/ip.h"
+#include "lwip/stack.h"
 
 #include "lwip/netif.h"
 #include "lwip/userfilter.h"
@@ -38,40 +39,11 @@
 #define NATPOSTS_DEBUG DBG_OFF
 #endif
 
-/*--------------------------------------------------------------------------*/
-
-#define TCP_MIN_PORT          0
-#define TCP_MAX_PORT          65535
-#define TCP_MAX_PORTS         (TCP_MAX_PORT - TCP_MIN_PORT)     
-#define TCP_PORTS_TABLE_SIZE  (TCP_MAX_PORTS / 8)
-
-unsigned char nat_tcp_table[TCP_PORTS_TABLE_SIZE];
-
-/*--------------------------------------------------------------------------*/
-
-#define UDP_MIN_PORT          0
-#define UDP_MAX_PORT          65535
-#define UDP_MAX_PORTS         (UDP_MAX_PORT - UDP_MIN_PORT)     
-#define UDP_PORTS_TABLE_SIZE  (UDP_MAX_PORTS / 8)
-
-unsigned char nat_udp_table[UDP_PORTS_TABLE_SIZE];
-
-/*--------------------------------------------------------------------------*/
-
-#define ICMP_MIN_PORT          0
-#define ICMP_MAX_PORT          65535
-#define ICMP_MAX_PORTS         (ICMP_MAX_PORT - ICMP_MIN_PORT)     
-#define ICMP_PORTS_TABLE_SIZE  (ICMP_MAX_PORTS / 8)
-
-unsigned char nat_icmp_table[ICMP_PORTS_TABLE_SIZE];
-
-/*--------------------------------------------------------------------------*/
-
-void nat_ports_init(void)
+void nat_ports_init(struct stack *stack)
 {
-	memset(nat_tcp_table , 0xff, TCP_PORTS_TABLE_SIZE);
-	memset(nat_udp_table , 0xff, UDP_PORTS_TABLE_SIZE);
-	memset(nat_icmp_table, 0xff, ICMP_PORTS_TABLE_SIZE);
+	memset(stack->stack_nat->nat_tcp_table , 0xff, TCP_PORTS_TABLE_SIZE);
+	memset(stack->stack_nat->nat_udp_table , 0xff, UDP_PORTS_TABLE_SIZE);
+	memset(stack->stack_nat->nat_icmp_table, 0xff, ICMP_PORTS_TABLE_SIZE);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -83,6 +55,7 @@ int  getnew_range(u16_t *val, unsigned char *table, u32_t start, u32_t range_min
 	range_min -= start;
 	range_max -= start;
 	
+	if (range_min < 0) range_min = 0;
 
 	for (i=range_min; i < range_max; i++)
 		if (table[ i/8 ]  &  (0x01 << (i % 8)))
@@ -101,26 +74,23 @@ int  getnew_range(u16_t *val, unsigned char *table, u32_t start, u32_t range_min
 	return 0;
 }
 
-int  nat_ports_getnew(int protocol, u16_t *port, u32_t min, u32_t max)
+int  nat_ports_getnew(struct stack *stack, int protocol, u16_t *port, u32_t min, u32_t max)
 {
 	u32_t r;
 
 	switch (protocol) {
-        	case IP_PROTO_ICMP4:
+		case IP_PROTO_ICMP4:
 		case IP_PROTO_ICMP:
-			//r = getnew( &nat_icmp_table[0], ICMP_PORTS_TABLE_SIZE) + ICMP_MIN_PORT;
-			r = getnew_range(port,  &nat_icmp_table[0], ICMP_MIN_PORT, min, max);
+			r = getnew_range(port,  &stack->stack_nat->nat_icmp_table[0], ICMP_MIN_PORT, min, max);
 			break;
 		case IP_PROTO_TCP:
-			//r = getnew( &nat_tcp_table[0], TCP_PORTS_TABLE_SIZE) +	TCP_MIN_PORT;
-			r = getnew_range(port,  &nat_tcp_table[0], TCP_MIN_PORT, min, max);
+			r = getnew_range(port,  &stack->stack_nat->nat_tcp_table[0], TCP_MIN_PORT, min, max);
 			break;
 		case IP_PROTO_UDP:
-			//r = getnew(  &nat_udp_table[0], UDP_PORTS_TABLE_SIZE) +	UDP_MIN_PORT;
-			r = getnew_range(port,  &nat_udp_table[0], UDP_MIN_PORT, min, max);
+			r = getnew_range(port,  &stack->stack_nat->nat_udp_table[0], UDP_MIN_PORT, min, max);
 			break;
 		default:
-			//FIX:
+			r = -1;
 			break;
 	}
 	return r;
@@ -131,21 +101,21 @@ int  nat_ports_getnew(int protocol, u16_t *port, u32_t min, u32_t max)
 #define nat_ports_unset(table, min, n)    ( (table)[((n)-(min))/8] |=  (1<<(((n)-(min))%8))    )
 
 
-u16_t nat_ports_free(int protocol, u16_t port)
+u16_t nat_ports_free(struct stack *stack, int protocol, u16_t port)
 {
 	switch (protocol) {
-        	case IP_PROTO_ICMP4:
+		case IP_PROTO_ICMP4:
 		case IP_PROTO_ICMP:
-			nat_ports_unset(nat_icmp_table, ICMP_MIN_PORT, port);
+			nat_ports_unset(stack->stack_nat->nat_icmp_table, ICMP_MIN_PORT, port);
 			break;
 		case IP_PROTO_TCP:
-			nat_ports_unset(nat_tcp_table, TCP_MIN_PORT, port);
+			nat_ports_unset(stack->stack_nat->nat_tcp_table, TCP_MIN_PORT, port);
 			break;
 		case IP_PROTO_UDP:
-			nat_ports_unset(nat_udp_table, UDP_MIN_PORT, port);
+			nat_ports_unset(stack->stack_nat->nat_udp_table, UDP_MIN_PORT, port);
 			break;
 		default:
-			//FIX:
+			return -1;
 			break;
 	}
 	return 1;
